// Copyright (c) 2001 NetApplication
package com.netapplications.webforums_jsp.control;

import javax.servlet.*;
import javax.servlet.http.*;
import com.netapplications.webforums_jsp.domain.Topico;
import com.netapplications.webforums_jsp.domain.VistaMensaje;
import com.netapplications.webforums_jsp.domain.MensajeTree;
import com.netapplications.webforums_jsp.persist.*;
import com.netapplications.webforums_jsp.types.*;
import com.netapplications.webforums_jsp.util.CookieUtils;

public class GetAllMessagesOfTopicCommand extends Command {
  private String nextThreadView;
  private String nextFlatView;
  private boolean sameMode;

  public GetAllMessagesOfTopicCommand(String nextThreadView, String nextFlatView, boolean sameMode) {
    this.nextThreadView = nextThreadView;
    this.nextFlatView = nextFlatView;
    this.sameMode = sameMode;
  }

  public GetAllMessagesOfTopicCommand(String nextThreadView, String nextFlatView) {
    this(nextThreadView, nextFlatView, true);
  }

  public String execute(HttpServletRequest req)
    throws CommandException {
      throw new CommandException("GetAllMessagesOfTopicCommand: This call is not allowed");
  }

  public String execute(HttpServletRequest req, HttpServletResponse res)
    throws CommandException {
    HttpSession session = req.getSession(true);
    try {
      TopicoPersist tdb = TopicoPersist.getInstance();
      VistaMensajePersist vmdb = VistaMensajePersist.getInstance();
      int id_topico;
      synchronized (req) {
        if (req.getAttribute("id_topico")!=null) {
          id_topico = Integer.parseInt((String) req.getAttribute("id_topico"));
        } else {
          id_topico = Integer.parseInt(req.getParameter("id_topico"));
        }
      }
      Topico topico = tdb.get(id_topico);
      req.setAttribute("topico", topico);
    	MensajeTree mtree = vmdb.getMensajesTopicoTree(topico.getId_topico());
      req.setAttribute("arbolmensajes", mtree);
      String vmode = "";
      if (req.getParameter("view_mode")!=null) {
        vmode = req.getParameter("view_mode");
      }
      if (!(vmode.equalsIgnoreCase(ModoVisualizacion.ARBOL) || vmode.equalsIgnoreCase(ModoVisualizacion.PLANO)) && req.getAttribute("view_mode")!=null) {
        vmode = (String) req.getAttribute("view_mode");
      }
      if (!(vmode.equalsIgnoreCase(ModoVisualizacion.ARBOL) || vmode.equalsIgnoreCase(ModoVisualizacion.PLANO)) && CookieUtils.isFoundCookie(req, "view_mode")) {
        vmode = CookieUtils.getCookieValue(req, "view_mode");
      }
      if (!(vmode.equalsIgnoreCase(ModoVisualizacion.ARBOL) || vmode.equalsIgnoreCase(ModoVisualizacion.PLANO))) {
        vmode = ModoVisualizacion.ARBOL;
      }
      if (!sameMode) {
        vmode = ModoVisualizacion.alternaModo(vmode);
      }
      Cookie cookie = new Cookie("view_mode", vmode);
      cookie.setMaxAge(60*60*24*365); // un ao
      res.addCookie(cookie);
      String rolForo = Roles.NOT_LOGGED;
      if (session.getAttribute("role")!=null) {
        rolForo = (String) session.getAttribute("role");
      }
      if (rolForo.equals(Roles.NOT_LOGGED)) {
        req.setAttribute("webforums.hint", "You need to <A href=\"javascript:document.forma.cmd.value=\'go-login\';document.forma.submit();\">register</A> to be able to participate on this forum");
      } else if (rolForo.equals(Roles.NOT_MEMBER)) {
        req.setAttribute("webforums.hint", "You need to <A href=\"javascript:document.forma.cmd.value=\'go-subscribe\';document.forma.submit();\">subscribe</A> to be able to participate on this forum");
      } else if (rolForo.equals(Roles.SUSPENDED)) {
        req.setAttribute("webforums.hint", "Your subscription was suspended by one moderator of this forum");
      } else if (rolForo.equals(Roles.INACTIVE_MEMBER)) {
        req.setAttribute("webforums.hint", "You need to reactivate your subscription to be able to participate on this forum");
      }
      if (vmode.equalsIgnoreCase(ModoVisualizacion.ARBOL)) {
        return nextThreadView;
      } else {
        return nextFlatView;
      }
    }
    catch (NumberFormatException e) {
      throw new CommandException("GetAllMessagesOfTopicCommand: invalid ID");
    }
    catch (PersistException e) {
      throw new CommandException("GetAllMessagesOfTopicCommand: " + e.getMessage());
    }
  }
}
