// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.domain;

import java.util.*;

public class MensajeTree extends Object {
  private static class MensajeNulo implements IMensajeMinimo {
    public int getId_mensaje() {
      return 0;
    }
    public int getId_mensaje_original() {
      return 0;
    }
  }
  private IMensajeMinimo mensaje_primero;
  private Vector nodos;

  public MensajeTree(IMensajeMinimo mensaje) {
    if (mensaje == null) {
      this.mensaje_primero = new MensajeNulo();
    } else {
      this.mensaje_primero = mensaje;
    }
    nodos = new Vector(5,2);
  }

  public MensajeTree() {
    this(null);
  }

  public void add(IMensajeMinimo mensaje) throws NoSuchElementException {
    if (mensaje.getId_mensaje_original() == mensaje_primero.getId_mensaje()) {
      nodos.add(new MensajeTree(mensaje));
    } else {
      for (int i=0;i<nodos.size();i++) {
        try {
          ((MensajeTree)nodos.get(i)).add(mensaje);
          return;
        } catch (NoSuchElementException ignore) {}
      }
      throw new NoSuchElementException("Element (" + mensaje.getId_mensaje()
        +") has no parents in the tree");
    }
  }

  public Object[] toArray(Object[] a) {
    Collection mensajes = new ArrayList();
    // primero va el padre (si es que no es un mensaje nulo)
    if (mensaje_primero.getId_mensaje()>0) {
      mensajes.add(mensaje_primero);
    }
    // luego sus hijitos... y nietos, bisnietos, etc.
    for (int i=0;i<nodos.size();i++) {
      Object[] arr = ((MensajeTree) nodos.get(i)).toArray(a);
      for (int j=0;j<arr.length;j++) {
        mensajes.add(arr[j]);
      }
    }
    return (Object[])mensajes.toArray(a);
  }

  public Integer[] analyzeDepthArray() {
    return this.analyzeDepthArray(new Integer(0));
  }

  public Integer[] analyzeDepthArray(Integer currDeep) {
    Collection profundidades = new ArrayList();
    // primero va el padre (si es que no es un mensaje nulo)
    if (mensaje_primero.getId_mensaje()>0) {
      profundidades.add(currDeep);
    }
    // luego sus hijitos... y nietos, bisnietos, etc.
    for (int i=0;i<nodos.size();i++) {
      Integer[] arr = ((MensajeTree) nodos.get(i)).analyzeDepthArray(new Integer(currDeep.intValue()+1));
      for (int j=0;j<arr.length;j++) {
        profundidades.add(arr[j]);
      }
    }
    return (Integer[])profundidades.toArray(new Integer[0]);
  }

  public int[] depthArray() {
    int[] arr;
    Integer[] deep;
    deep = analyzeDepthArray();
    arr = new int[deep.length];
    for (int j=0;j<deep.length;j++) {
      arr[j] = deep[j].intValue();
    }
    return arr;
  }

  public static void main(String[] argc) {
    try {
      MensajeTree t = new MensajeTree();
      Mensaje b = new Mensaje();
      b.setId_mensaje(1);
      t.add(b);
      //MensajeTree t = new MensajeTree(b);
      b = new Mensaje();
      b.setId_mensaje(2);
      b.setId_mensaje_original(1);
      t.add(b);
      b = new Mensaje();
      b.setId_mensaje(3);
      b.setId_mensaje_original(2);
      t.add(b);
      b = new Mensaje();
      b.setId_mensaje(4);
      b.setId_mensaje_original(2);
      t.add(b);
      b = new Mensaje();
      b.setId_mensaje(5);
      b.setId_mensaje_original(1);
      t.add(b);
      b = new Mensaje();
      b.setId_mensaje(6);
      b.setId_mensaje_original(4);
      t.add(b);
      b = new Mensaje();
      b.setId_mensaje(7);
      b.setId_mensaje_original(2);
      t.add(b);
      b = new Mensaje();
      b.setId_mensaje(8);
      b.setId_mensaje_original(0);
      t.add(b);
      Mensaje[] a = (Mensaje[]) t.toArray(new Mensaje[0]);
      for (int j=0;j<a.length;j++) {
        System.out.println(a[j].getId_mensaje() + " - " + a[j].getId_mensaje_original());
      }
      int[] h = t.depthArray();
      for (int j=0;j<h.length;j++) {
        System.out.println(h[j]);
      }
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
