// Copyright (c) 2001 NetApplication
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import com.netapplications.webforums_jsp.util.Settings;

public class EmoticonIDGenerator {
  private int increment = 1;
  private int max = 0;
  private int currentID = 0;

  private Connection connection;
  private static EmoticonIDGenerator instance;
  private PreparedStatement updSurrogateStmt;
  private PreparedStatement getSurrogateStmt;

  public static EmoticonIDGenerator getInstance() throws PersistException {
    if (instance == null) {
      instance = new EmoticonIDGenerator();
    }
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getSurrogateStmt = connection.prepareStatement(Settings.get("sql.EmoticonIDGenerator.get"));
    updSurrogateStmt = connection.prepareStatement(Settings.get("sql.EmoticonIDGenerator.upd"));
  }

  private EmoticonIDGenerator() throws PersistException {
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("EmoticonIDGenerator [EmoticonIDGenerator()]: " + se.getMessage());
    }
  }

  public synchronized int getNextID() throws PersistException {
    currentID++;
  	if (currentID > max) {
      Connection confrompool = null;
      try {
        confrompool = ConnectionPool.get();
        if (confrompool != connection) {
          synchronized (connection) {
            connection = confrompool;
          }
          parseStatements();
        }
        synchronized(updSurrogateStmt) {
          updSurrogateStmt.clearParameters();
          updSurrogateStmt.setInt(1, increment);
          int rowsChanged = updSurrogateStmt.executeUpdate();
          if (rowsChanged < 1) {
            throw new UnknownPersistException("EmoticonIDGenerator (getNextID()): " + " Failed to update Surrogates");
          }
        }
        synchronized (getSurrogateStmt) {
          ResultSet results = getSurrogateStmt.executeQuery();
          if (results.next()) {
            currentID = results.getInt("NEXTIDSET") - increment;
            max = currentID + increment - 1;
          } else {
            throw new UnknownPersistException("EmoticonIDGenerator [getNextID()]: " + " Failed to read Surrogates");
          }
        }
      } catch (SQLException e) {
        throw new PersistException("EmoticonIDGenerator [getNextID()]: " + e.getMessage());
      }
      finally {
        if (confrompool!=null) {
          ConnectionPool.free(confrompool);
        }
      }
    }
    return currentID;
  }
}
