// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.util.*;
import java.sql.*;
import com.netapplications.webforums_jsp.util.Settings;
import com.netapplications.webforums_jsp.domain.Emoticon;

public class EmoticonPersist extends Object {
  private static EmoticonPersist instance;

  private Connection connection;
  private PreparedStatement getAllEmoticonsStmt;
  private PreparedStatement putEmoticonStmt;
  private PreparedStatement updEmoticonStmt;
  private PreparedStatement remEmoticonStmt;
  private PreparedStatement remEmoticonsMensStmt;
  private EmoticonIDGenerator idgen;

  public static EmoticonPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new EmoticonPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getAllEmoticonsStmt = connection.prepareStatement(Settings.get("sql.EmoticonPersist.getAllEmoticons"));
    putEmoticonStmt = connection.prepareStatement(Settings.get("sql.EmoticonPersist.putEmoticon"));
    updEmoticonStmt = connection.prepareStatement(Settings.get("sql.EmoticonPersist.updEmoticon"));
    remEmoticonStmt = connection.prepareStatement(Settings.get("sql.EmoticonPersist.remEmoticon"));
    remEmoticonsMensStmt = connection.prepareStatement(Settings.get("sql.EmoticonPersist.remEmoticonsMens"));
  }

  private EmoticonPersist() throws PersistException {
    idgen = EmoticonIDGenerator.getInstance();
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("EmoticonPersist [EmoticonPersist()]: " + se.getMessage());
    }
  }

  private Emoticon makeEmoticon(ResultSet results)
    throws PersistException {
    try {
      Emoticon emot = new Emoticon();
      emot.setId_emoticon(results.getInt("ID_EMOTICON"));
      emot.setUri(results.getString("URI"));
      return emot;
    }
    catch (SQLException e) {
      throw new PersistException("EmoticonPersist [makeEmoticon()]: " + e.getMessage());
    }
  }

  public Emoticon[] getAllEmoticons()
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection Emoticons = new ArrayList();
      synchronized(getAllEmoticonsStmt) {
        results = getAllEmoticonsStmt.executeQuery();
      }
      while (results.next()) {
        Emoticons.add(makeEmoticon(results));
      }
      return (Emoticon[])Emoticons.toArray(new Emoticon[0]);
    }
    catch (SQLException e) {
      throw new PersistException("EmoticonPersist (getAllEmoticons()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void update(Emoticon emot)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updEmoticonStmt) {
        updEmoticonStmt.clearParameters();
        updEmoticonStmt.setString(1, emot.getUri());
        updEmoticonStmt.setInt(2, emot.getId_emoticon());
        int rowsChanged = updEmoticonStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("EmoticonPersist [updateEmoticon()]: " + "No se encontro el Emoticon # " +
              emot.getId_emoticon());
      }
    }
    catch (SQLException e) {
      throw new PersistException("EmoticonPersist [update()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void put(Emoticon emot) throws
    PersistException {
    int new_id = idgen.getNextID();
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putEmoticonStmt) {
        putEmoticonStmt.clearParameters();
        emot.setId_emoticon(new_id);
        putEmoticonStmt.setInt(1, emot.getId_emoticon());
        putEmoticonStmt.setString(2, emot.getUri());
        putEmoticonStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("EmoticonPersist [putEmoticon()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  private void remEmoticonsMensajes(int id_emoticon)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(remEmoticonsMensStmt) {
        remEmoticonsMensStmt.clearParameters();
        remEmoticonsMensStmt.setInt(1, id_emoticon);
        remEmoticonsMensStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("EmoticonPersist [remEmoticonsMensajes()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void removeEmoticonCascade(int id_emoticon)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      remEmoticonsMensajes(id_emoticon);
      synchronized(remEmoticonStmt) {
        remEmoticonStmt.clearParameters();
        remEmoticonStmt.setInt(1, id_emoticon);
        int rowsChanged = remEmoticonStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("EmoticonPersist: " + "No se pudo borrar el Emoticon # "
            + id_emoticon);
      }
    }
    catch (SQLException e) {
      throw new PersistException("EmoticonPersist [removeEmoticon()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }


  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      Settings.getInstance().load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      System.out.println(Settings.get("sql.EmoticonPersist.getAllEmoticons"));
      System.out.println(Settings.get("sql.EmoticonIDGenerator.upd"));
      System.out.println(Settings.get("sql.EmoticonIDGenerator.get"));
      System.out.println(Settings.get("sql.ForoIDGenerator.upd"));
      System.out.println(Settings.get("sql.MensajePersist.updContenidoMensaje"));
      EmoticonPersist edb = EmoticonPersist.getInstance();
      Emoticon[] em = edb.getAllEmoticons();
      for (int i=1;i<em.length;i++) {
        System.out.println(em[i].getUri());
      }
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
