// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.util.*;
import java.sql.*;
import com.netapplications.webforums_jsp.domain.Foro;
import com.netapplications.webforums_jsp.util.Settings;
import com.netapplications.webforums_jsp.util.FileUtils;

public class ForoPersist extends Object {

  private static ForoPersist instance;

  private Connection connection;
  private PreparedStatement getForoStmt;
  private PreparedStatement putForoStmt;
  private PreparedStatement remForoStmt;
  private PreparedStatement getAllForosUsuarioStmt;
  private PreparedStatement updForoStmt;
  private PreparedStatement visitaForoStmt;
  private PreparedStatement getAllForosActivosStmt;
  private PreparedStatement incNumTopicosForoStmt;
  private PreparedStatement decNumTopicosForoStmt;
  private PreparedStatement getCountTopicosStmt;
  private PreparedStatement updCountTopicosStmt;


  private ForoIDGenerator foridgen;

  public static ForoPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new ForoPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.getForo"));
    putForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.putForo"));
    updForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.updForo"));
    remForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.remForo"));
    getAllForosUsuarioStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.allForoUsuario"));
    getAllForosActivosStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.allForoActivos"));
    visitaForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.visitaForo"));
    incNumTopicosForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.incNumTopicosForo"));
    decNumTopicosForoStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.decNumTopicosForo"));
    getCountTopicosStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.getCountTopicos"));
    updCountTopicosStmt = connection.prepareStatement(Settings.get("sql.ForoPersist.updCountTopicos"));
  }

  private ForoPersist() throws PersistException {
    foridgen = ForoIDGenerator.getInstance();
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("ForoPersist (ForoPersist()): " + se.getMessage());
    }
  }

  private Foro makeForo(ResultSet results)
    throws PersistException {
    try {
      Foro Foro = new Foro();
      Foro.setId_foro(results.getInt("ID_FORO"));
      Foro.setNombre(results.getString("NOMBRE"));
      Foro.setDescripcion(results.getString("DESCRIPCION"));
      Foro.setEs_activo(results.getString("ES_ACTIVO"));
      Foro.setFecha_creacion((java.util.Date) results.getTimestamp("FECHA_CREACION"));
      Foro.setNum_visitas(results.getInt("NUM_VISITAS"));
      Foro.setFecha_ultima_visita((java.util.Date) results.getTimestamp("FECHA_ULTIMA_VISITA"));
      Foro.setNum_topicos(results.getInt("NUM_TOPICOS"));
      return Foro;
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (makeForo()): " + e.getMessage());
    }
  }

  public void fixCounts() throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized(getCountTopicosStmt) {
        results = getCountTopicosStmt.executeQuery();
      }
      synchronized(updCountTopicosStmt) {
        while (results.next()) {
          updCountTopicosStmt.clearParameters();
          updCountTopicosStmt.setInt(1, results.getInt(2));
          updCountTopicosStmt.setInt(2, results.getInt(1));
          updCountTopicosStmt.executeUpdate();
        }
      }
    } catch (SQLException e) {
      throw new PersistException("ForoPersist (fixCounts()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Foro get(int id_foro)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getForoStmt) {
        getForoStmt.clearParameters();
        getForoStmt.setInt(1, id_foro);
        results = getForoStmt.executeQuery();
      }
      if (results.next())
        return makeForo(results);
      else
        throw new UnknownPersistException("ForoPersist (getForo()): " + "No se encontro el Foro # " + id_foro);
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (getForo()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Foro[] getForosUsuario(int id_usuario)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection Foros = new ArrayList();
      synchronized(getAllForosUsuarioStmt) {
        getAllForosUsuarioStmt.clearParameters();
        getAllForosUsuarioStmt.setInt(1, id_usuario);
        results = getAllForosUsuarioStmt.executeQuery();
      }
      while (results.next()) {
        Foros.add(makeForo(results));
      }
      return (Foro[])Foros.toArray(new Foro[0]);
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (getForosUsuario()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Foro[] getForosActivos()
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection Foros = new ArrayList();
      synchronized(getAllForosActivosStmt) {
        results = getAllForosActivosStmt.executeQuery();
      }
      while (results.next()) {
        Foros.add(makeForo(results));
      }
      Foro[] ret = (Foro[])Foros.toArray(new Foro[0]);
      return ret;
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (getForosActivos()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void update(Foro Foro)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updForoStmt) {
        updForoStmt.clearParameters();
        updForoStmt.setString(1, Foro.getNombre());
        updForoStmt.setString(2, Foro.getDescripcion());
        updForoStmt.setString(3, Foro.getEs_activo());
        updForoStmt.setTimestamp(4, new java.sql.Timestamp(Foro.getFecha_creacion().getTime()));
        updForoStmt.setInt(5, Foro.getNum_visitas());
        updForoStmt.setTimestamp(6, new java.sql.Timestamp(Foro.getFecha_ultima_visita().getTime()));
        updForoStmt.setInt(7, Foro.getNum_topicos());
        updForoStmt.setInt(8, Foro.getId_foro());
        int rowsChanged = updForoStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("ForoPersist (update()): " + "No se encontro el Foro # " +
              Foro.getId_foro());
      }
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (update()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void put(Foro Foro) throws PersistException {
    int new_id = foridgen.getNextID();
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putForoStmt) {
        putForoStmt.clearParameters();
        Foro.setId_foro(new_id);
        putForoStmt.setInt(1, Foro.getId_foro());
        putForoStmt.setString(2, Foro.getNombre());
        putForoStmt.setString(3, Foro.getDescripcion());
        putForoStmt.setString(4, Foro.getEs_activo());
        putForoStmt.setTimestamp(5, new java.sql.Timestamp(Foro.getFecha_creacion().getTime()));
        putForoStmt.setInt(6, Foro.getNum_visitas());
        putForoStmt.setTimestamp(7, new java.sql.Timestamp(Foro.getFecha_ultima_visita().getTime()));
        putForoStmt.setInt(8, Foro.getNum_topicos());
        putForoStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (put()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void remove(int id_foro) throws PersistException {
/****************************** Borrar Detalle ******************************/
    TopicoPersist.getInstance().removeAllTopicosForo(id_foro);
    SuscripcionPersist.getInstance().removeSuscripcionesForo(id_foro);
/****************************************************************************/
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(remForoStmt) {
        remForoStmt.clearParameters();
        remForoStmt.setInt(1, id_foro);
        int rowsChanged = remForoStmt.executeUpdate();
        if (rowsChanged < 1) {
          throw new UnknownPersistException("ForoPersist: (remove()) "
            + "No se pudo borrar el Foro # " + id_foro);
        } else {
          FileUtils.delete(Settings.get("servlet.attachmentRealPath")+id_foro);
        }
      }
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist: (remove())" + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void visitaForo(int id_foro) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(visitaForoStmt) {
        visitaForoStmt.clearParameters();
        visitaForoStmt.setTimestamp(1, new java.sql.Timestamp(new java.util.Date().getTime()));
        visitaForoStmt.setInt(2, id_foro);
        int rowsChanged = visitaForoStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("ForoPersist (visitaForo()): " + "No se pudo incrementar las visitas del Foro # "
            + id_foro);
      }
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (visitaForo()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void incrementNumTopicosForo(int id_foro) throws PersistException {
    incrementNumTopicosForo(id_foro, 1);
  }

  public void incrementNumTopicosForo(int id_foro, int increment) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(incNumTopicosForoStmt) {
        incNumTopicosForoStmt.clearParameters();
        incNumTopicosForoStmt.setInt(1, increment);
        incNumTopicosForoStmt.setInt(2, id_foro);
        int rowsChanged = incNumTopicosForoStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("ForoPersist (incrementNumTopicosForo()): " + "No se pudo incrementar el contador de "
            + "topicos del Foro # " + id_foro);
      }
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (incrementNumTopicosForo()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }


  public void decrementNumTopicosForo(int id_foro) throws PersistException {
    decrementNumTopicosForo(id_foro, 1);
  }

  public void decrementNumTopicosForo(int id_foro, int decrement) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(decNumTopicosForoStmt) {
        decNumTopicosForoStmt.clearParameters();
        decNumTopicosForoStmt.setInt(1, decrement);
        decNumTopicosForoStmt.setInt(2, id_foro);
        decNumTopicosForoStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("ForoPersist (decrementNumTopicosForo()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      ForoPersist idb = ForoPersist.getInstance();
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
