// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.Mensaje;
import com.netapplications.webforums_jsp.domain.Attachment;
import com.netapplications.webforums_jsp.util.Settings;
import com.netapplications.webforums_jsp.util.FileUtils;

public class MensajePersist extends Object {
  private static MensajePersist instance;

  private Connection connection;
  private PreparedStatement getMensajeStmt;
  private PreparedStatement putMensajeStmt;
  private PreparedStatement remMensajeStmt;
  private PreparedStatement updMensajeStmt;
  private PreparedStatement putAttachmentStmt;
  private PreparedStatement getIdMensajesTopicoStmt;
  private PreparedStatement getIdMensajesHijosStmt;
  private PreparedStatement addRespuestaTopicoStmt;
  private PreparedStatement addTopicoForumStmt;
  private PreparedStatement remRespuestaTopicoStmt;
  private PreparedStatement remTopicoForumStmt;
  private PreparedStatement getNextTopicoStmt;
  private PreparedStatement getPrevTopicoStmt;
  private PreparedStatement getAttachmentStmt;
  private PreparedStatement remAttachmentStmt;
  private PreparedStatement updContenidoMensajeStmt;

  private MensajeIDGenerator mensidgen;

  public static MensajePersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new MensajePersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    updContenidoMensajeStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.updContenidoMensaje"));
    getAttachmentStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.getAttachment"));
    remAttachmentStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.remAttachment"));
    putAttachmentStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.putAttachment"));
    getMensajeStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.getMensaje"));
    putMensajeStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.putMensaje"));
    remMensajeStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.remMensaje"));
    updMensajeStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.updMensaje"));
    getIdMensajesTopicoStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.getIdMensajesTopico"));
    getIdMensajesHijosStmt = connection.prepareStatement(Settings.get("sql.MensajePersist.getIdMensajesHijos"));
  }

  private MensajePersist() throws PersistException {
    mensidgen = MensajeIDGenerator.getInstance();
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("MensajePersist [MensajePersist()]: " + se.getMessage());
    }
  }

  private Mensaje makeMensaje(ResultSet results)
    throws PersistException {
    try {
      Mensaje mens = new Mensaje();
      mens.setId_mensaje(results.getInt("ID_MENSAJE"));
      mens.setId_topico(results.getInt("ID_TOPICO"));
      mens.setId_mensaje_original(results.getInt("ID_MENSAJE_ORIGINAL"));
      mens.setAsunto(results.getString("ASUNTO"));
      mens.setContenido(results.getString("CONTENIDO"));
      mens.setPrioridad(results.getInt("PRIORIDAD"));
      mens.setFecha_envio((java.util.Date) results.getTimestamp("FECHA_ENVIO"));
      mens.setEnvio_anonimo(results.getString("ENVIO_ANONIMO"));
      mens.setEstatus(results.getString("ESTATUS"));
      mens.setIncluir_original(results.getString("INCLUIR_ORIGINAL"));
      mens.setId_foro(results.getInt("ID_FORO"));
      mens.setId_usuario(results.getInt("ID_USUARIO"));
      mens.setId_emoticon(results.getInt("ID_EMOTICON"));
      mens.setNombre_autor(results.getString("NOMBRE_AUTOR"));
      return mens;
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [makeMensaje()]: " + e.getMessage());
    }
  }

  public Mensaje get(int id_mensaje)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getMensajeStmt) {
        getMensajeStmt.clearParameters();
        getMensajeStmt.setInt(1, id_mensaje);
        results = getMensajeStmt.executeQuery();
      }
      if (results.next())
        return makeMensaje(results);
      else
        throw new UnknownPersistException("MensajePersist [getMensaje()]: " + "No se encontro el mens # " + id_mensaje);
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [getMensaje()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void update(Mensaje mens)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updMensajeStmt) {
        updMensajeStmt.clearParameters();
        updMensajeStmt.setInt(1, mens.getId_topico());
        if (mens.getId_mensaje_original()>0) {
          updMensajeStmt.setInt(2, mens.getId_mensaje_original());
        } else {
          updMensajeStmt.setNull(2, java.sql.Types.INTEGER);
        }
        updMensajeStmt.setString(3, mens.getAsunto());
        updMensajeStmt.setString(4, mens.getContenido());
        updMensajeStmt.setInt(5, mens.getPrioridad());
        updMensajeStmt.setTimestamp(6, new java.sql.Timestamp(mens.getFecha_envio().getTime()));
        updMensajeStmt.setString(7, mens.getEnvio_anonimo());
        updMensajeStmt.setString(8, mens.getEstatus());
        updMensajeStmt.setString(9, mens.getIncluir_original());
        updMensajeStmt.setInt(10, mens.getId_foro());
        updMensajeStmt.setInt(11, mens.getId_usuario());
        if (mens.getId_emoticon()>0) {
          updMensajeStmt.setInt(12, mens.getId_emoticon());
        } else {
          updMensajeStmt.setNull(12, java.sql.Types.INTEGER);
        }
        if (mens.getNombre_autor()!=null && mens.getNombre_autor().length()>0) {
          updMensajeStmt.setString(13, mens.getNombre_autor());
        } else {
          updMensajeStmt.setNull(13, java.sql.Types.VARCHAR);
        }
        updMensajeStmt.setInt(14, mens.getId_mensaje());
        int rowsChanged = updMensajeStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("MensajePersist [updateMensaje()]: " + "No se encontro el mens # " +
              mens.getId_mensaje());
      }
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [updateMensaje()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void updateContenidoMensaje(int id_mensaje, String contenido)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updContenidoMensajeStmt) {
        updContenidoMensajeStmt.clearParameters();
        updContenidoMensajeStmt.setString(1, contenido);
        updContenidoMensajeStmt.setInt(2, id_mensaje);
        int rowsChanged = updContenidoMensajeStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("MensajePersist [updateContenidoMensaje()]: " + "No se encontro el mens # " +
              id_mensaje);
      }
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [updateContenidoMensaje()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }


  public void putAttachment(Attachment attach) throws
    PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putAttachmentStmt) {
        putAttachmentStmt.clearParameters();
        putAttachmentStmt.setInt(1, attach.getId_mensaje());
        putAttachmentStmt.setString(2, attach.getUri());
        putAttachmentStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [putAttachment()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void put(Mensaje mens) throws PersistException {
    boolean done = false;
    int new_id = mensidgen.getNextID();
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putMensajeStmt) {
        putMensajeStmt.clearParameters();
        mens.setId_mensaje(new_id);
        putMensajeStmt.setInt(1, mens.getId_mensaje());
        putMensajeStmt.setInt(2, mens.getId_topico());
        if (mens.getId_mensaje_original()>0) {
          putMensajeStmt.setInt(3, mens.getId_mensaje_original());
        } else {
          putMensajeStmt.setNull(3, java.sql.Types.INTEGER);
        }
        putMensajeStmt.setString(4, mens.getAsunto());
        putMensajeStmt.setString(5, mens.getContenido());
        putMensajeStmt.setInt(6, mens.getPrioridad());
        putMensajeStmt.setTimestamp(7, new java.sql.Timestamp(mens.getFecha_envio().getTime()));
        putMensajeStmt.setString(8, mens.getEnvio_anonimo());
        putMensajeStmt.setString(9, mens.getEstatus());
        putMensajeStmt.setString(10, mens.getIncluir_original());
        putMensajeStmt.setInt(11, mens.getId_foro());
        putMensajeStmt.setInt(12, mens.getId_usuario());
        if (mens.getId_emoticon()>0) {
          putMensajeStmt.setInt(13, mens.getId_emoticon());
        } else {
          putMensajeStmt.setNull(13, java.sql.Types.INTEGER);
        }
        if (mens.getNombre_autor()!=null && mens.getNombre_autor().length()>0) {
          putMensajeStmt.setString(14, mens.getNombre_autor());
        } else {
          putMensajeStmt.setNull(14, java.sql.Types.VARCHAR);
        }
        putMensajeStmt.executeUpdate();
      }
      done = true;
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [putMensaje()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
    if (done) {
      TopicoPersist.getInstance().incrementNumMensajesTopico(mens.getId_topico());
    }
  }

  public void removeAllMensajesTopico(int id_topico) throws PersistException {
    ResultSet results;
    Connection confrompool = null;
    Integer[] arrIds = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(getIdMensajesTopicoStmt) {
        getIdMensajesTopicoStmt.clearParameters();
        getIdMensajesTopicoStmt.setInt(1, id_topico);
        arrIds = getIntegerArray(getIdMensajesTopicoStmt.executeQuery(), 1);
      }
      ConnectionPool.free(confrompool);
      confrompool = null;
      for (int i=0; i<arrIds.length; i++) {
        removeMensajeCascade(arrIds[i].intValue());
      }
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [removeAllMensajesTopico()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  private Integer[] getIntegerArray(ResultSet results, int position) throws SQLException {
    Collection ints = new ArrayList();
    while (results.next()) {
      ints.add(new Integer(results.getInt(position)));
    }
    return (Integer[])ints.toArray(new Integer[0]);
  }

  private static boolean removeFile(String pathname) {
    return FileUtils.delete(Settings.get("servlet.attachmentRealPath") + pathname)
      && FileUtils.delete(Settings.get("servlet.attachmentRealPath") + pathname.substring(0, pathname.lastIndexOf("/")));
  }

  private boolean removeAttachment(int id_mensaje)
    throws PersistException {
    ResultSet results;
    try {
      synchronized(getAttachmentStmt) {
        getAttachmentStmt.clearParameters();
        getAttachmentStmt.setInt(1, id_mensaje);
        results = getAttachmentStmt.executeQuery();
      }
      if (!results.next()) {
        return true;
      } else {
        String path = results.getString(1);
        if (removeFile(path)) {
          synchronized(remAttachmentStmt) {
            remAttachmentStmt.setInt(1, id_mensaje);
            return remAttachmentStmt.executeUpdate()>0;
          }
        } else {
          throw new PersistException("MensajePersist [removeAttachment()]: Failed to remove attachment "
            + path);
        }
      }
    } catch (Exception e) {
      throw new PersistException("MensajePersist [removeAttachment()]: " + e.getMessage());
    }
  }

  public int removeMensajeCascade(int id_mensaje) throws PersistException {
    Integer[] arrIds = null;
    ResultSet results;
    int retVal = 0;
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
// En este bloque obtendremos todos los mensajes que son hijos directos
// de este mensaje (responden al mismo)
      synchronized(getIdMensajesHijosStmt) {
        getIdMensajesHijosStmt.clearParameters();
        getIdMensajesHijosStmt.setInt(1, id_mensaje);
        arrIds = getIntegerArray(getIdMensajesHijosStmt.executeQuery(), 1);
      }
// Recursivamente llamaremos a esta misma funcin para borrar antes
// a estos mensajes hijos, pero antes de hacerlo liberamos la coneccin
// en el pool
      ConnectionPool.free(confrompool);
      confrompool = null;
      for (int i=0; i<arrIds.length; i++) {
        retVal += removeMensajeCascade(arrIds[i].intValue());
      }
// Al llegar a este punto ya no debe tener este mensaje ningn mensaje
// dependiente por lo tanto es momento de borrarlo a l, para ello
// requerimos otra vez una coneccin del pool
        confrompool = ConnectionPool.get();
        if (confrompool != connection) {
          synchronized (connection) {
            connection = confrompool;
          }
          parseStatements();
        }
// Primero debemos borrar el attachment (si lo hay) y luego el mensaje en s
      if (removeAttachment(id_mensaje)) {
        if (deleteMensaje(id_mensaje) < 1) {
          throw new UnknownPersistException("MensajePersist [removeMensajeCascade()]: " + "No se pudo borrar el mens # "
              + id_mensaje);
        } else {
          return ++retVal;
        }
      } else {
        throw new PersistException("MensajePersist [removeMensajeCascade()]: Failed to delete attachment");
      }
    } catch (SQLException e) {
      throw new PersistException("MensajePersist [removeMensajeCascade()]: " + e.getMessage());
    } finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void removeMensaje(int id_mensaje, int id_topico) throws PersistException {
    boolean done = false;
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      if (deleteMensaje(id_mensaje) < 1) {
        throw new UnknownPersistException("MensajePersist [removeMensaje()]: " + "No se pudo borrar el mens # "
            + id_mensaje);
      }
    }
    catch (SQLException e) {
      throw new PersistException("MensajePersist [removeMensaje()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
    if (done) {
      TopicoPersist.getInstance().decrementNumMensajesTopico(id_topico);
    }
  }

  private int deleteMensaje(int id_mensaje) throws SQLException {
    synchronized(remMensajeStmt) {
      remMensajeStmt.clearParameters();
      remMensajeStmt.setInt(1, id_mensaje);
      return remMensajeStmt.executeUpdate();
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      Settings.getInstance().load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      MensajePersist mdb = MensajePersist.getInstance();
/*
      Mensaje msg = new Mensaje();
      msg.setAsunto("prueba");
      msg.setContenido("prueba");
      msg.setEnvio_anonimo("N");
      msg.setEstatus("A");
      msg.setFecha_envio(new java.util.Date());
      msg.setId_usuario(1);
      msg.setId_foro(14);
      msg.setId_topico(20);
      mdb.put(msg);
*/
    mdb.removeAllMensajesTopico(92);
//    mdb.removeMensajeCascade(298);
  } catch (Exception e) {System.out.println(e.getMessage());}
  }
}
