// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.util.*;
import java.sql.*;
import com.netapplications.webforums_jsp.domain.PerfilUsuario;
import com.netapplications.webforums_jsp.util.Settings;

public class PerfilUsuarioPersist extends Object {
  private static PerfilUsuarioPersist instance;

  private Connection connection;
  private PreparedStatement getPerfilUsuarioStmt;
  private PreparedStatement putPerfilUsuarioStmt;
  private PreparedStatement remPerfilUsuarioStmt;
  private PreparedStatement updPerfilUsuarioStmt;

  public static PerfilUsuarioPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new PerfilUsuarioPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getPerfilUsuarioStmt = connection.prepareStatement(Settings.get("sql.PerfilUsuarioPersist.get"));
    putPerfilUsuarioStmt = connection.prepareStatement(Settings.get("sql.PerfilUsuarioPersist.put"));
    remPerfilUsuarioStmt = connection.prepareStatement(Settings.get("sql.PerfilUsuarioPersist.rem"));
    updPerfilUsuarioStmt = connection.prepareStatement(Settings.get("sql.PerfilUsuarioPersist.upd"));
  }

  private PerfilUsuarioPersist() throws PersistException {
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("PerfilUsuarioPersist [PerfilUsuarioPersist()]: " + se.getMessage());
    }
  }

  private PerfilUsuario makePerfilUsuario(ResultSet results)
    throws PersistException {
    try {
      PerfilUsuario perfil = new PerfilUsuario();
      perfil.setId_usuario(results.getInt("ID_USUARIO"));
      perfil.setTitulo(results.getString("TITULO"));
      perfil.setCompania(results.getString("COMPANIA"));
      perfil.setDireccion1(results.getString("DIRECCION1"));
      perfil.setDireccion2(results.getString("DIRECCION2"));
      perfil.setCiudad(results.getString("CIUDAD"));
      perfil.setEstado(results.getString("ESTADO"));
      perfil.setCp(results.getString("CP"));
      perfil.setTelefono(results.getString("TELEFONO"));
      perfil.setUri(results.getString("URI"));
      return perfil;
    }
    catch (SQLException e) {
      throw new PersistException("PerfilUsuarioPersist [makePerfilUsuario()]: " + e.getMessage());
    }
  }

  public PerfilUsuario get(int id_usuario)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getPerfilUsuarioStmt) {
        getPerfilUsuarioStmt.clearParameters();
        getPerfilUsuarioStmt.setInt(1, id_usuario);
        results = getPerfilUsuarioStmt.executeQuery();
      }
      if (results.next())
        return makePerfilUsuario(results);
      else
        throw new UnknownPersistException("PerfilUsuarioPersist [get()]: " + "No se encontro el Perfil del Usuario # " + id_usuario);
    }
    catch (SQLException e) {
      throw new PersistException("PerfilUsuarioPersist [get()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }


  public void update(PerfilUsuario perfil)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updPerfilUsuarioStmt) {
        updPerfilUsuarioStmt.clearParameters();
        updPerfilUsuarioStmt.setString(1, perfil.getTitulo());
        updPerfilUsuarioStmt.setString(2, perfil.getCompania());
        updPerfilUsuarioStmt.setString(3, perfil.getDireccion1());
        updPerfilUsuarioStmt.setString(4, perfil.getDireccion2());
        updPerfilUsuarioStmt.setString(5, perfil.getCiudad());
        updPerfilUsuarioStmt.setString(6, perfil.getEstado());
        updPerfilUsuarioStmt.setString(7, perfil.getCp());
        updPerfilUsuarioStmt.setString(8, perfil.getTelefono());
        updPerfilUsuarioStmt.setString(9, perfil.getUri());
        updPerfilUsuarioStmt.setInt(10, perfil.getId_usuario());
        int rowsChanged = updPerfilUsuarioStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("PerfilUsuarioPersist [update()]: " + "No se encontro el Perfil del Usuario # " +
              perfil.getId_usuario());
      }
    }
    catch (SQLException e) {
      throw new PersistException("PerfilUsuarioPersist [update()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void put(PerfilUsuario perfil) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putPerfilUsuarioStmt) {
        putPerfilUsuarioStmt.clearParameters();
        putPerfilUsuarioStmt.setInt(1, perfil.getId_usuario());
        putPerfilUsuarioStmt.setString(2, perfil.getTitulo());
        putPerfilUsuarioStmt.setString(3, perfil.getCompania());
        putPerfilUsuarioStmt.setString(4, perfil.getDireccion1());
        putPerfilUsuarioStmt.setString(5, perfil.getDireccion2());
        putPerfilUsuarioStmt.setString(6, perfil.getCiudad());
        putPerfilUsuarioStmt.setString(7, perfil.getEstado());
        putPerfilUsuarioStmt.setString(8, perfil.getCp());
        putPerfilUsuarioStmt.setString(9, perfil.getTelefono());
        putPerfilUsuarioStmt.setString(10, perfil.getUri());
        putPerfilUsuarioStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("PerfilUsuarioPersist [put()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void remove(int id) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(remPerfilUsuarioStmt) {
        remPerfilUsuarioStmt.clearParameters();
        remPerfilUsuarioStmt.setInt(1, id);
        int rowsChanged = remPerfilUsuarioStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("PerfilUsuarioPersist: " + "No se pudo borrar el Perfil del Usuario # "
            + id);
      }
    }
    catch (SQLException e) {
      throw new PersistException("PerfilUsuarioPersist [remove()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      PerfilUsuarioPersist pudb = PerfilUsuarioPersist.getInstance();
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
