// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.util.*;
import java.sql.*;
import com.netapplications.webforums_jsp.domain.Suscripcion;
import com.netapplications.webforums_jsp.types.*;
import com.netapplications.webforums_jsp.control.Roles;
import com.netapplications.webforums_jsp.util.Settings;

public class SuscripcionPersist extends Object {
  private static SuscripcionPersist instance;

  private Connection connection;
  private PreparedStatement getTipoSuscripcionStmt;
  private PreparedStatement getSuscripcionStmt;
  private PreparedStatement putSuscripcionStmt;
  private PreparedStatement remSuscripcionStmt;
  private PreparedStatement getAllSuscripcionesUsuarioStmt;
  private PreparedStatement updSuscripcionStmt;
  private PreparedStatement visitasuscripStmt;
  private PreparedStatement remAllSuscripcionesForumStmt;
  private PreparedStatement updTipoSuscripcionStmt;
  private PreparedStatement visitaSuscripcionStmt;

  public static SuscripcionPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new SuscripcionPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    remAllSuscripcionesForumStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.remAllSuscripcionesForum"));
    getTipoSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.getMembresiaForum"));
    getSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.getsuscrip"));
    putSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.putsuscrip"));
    remSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.remSuscripcion"));
    getAllSuscripcionesUsuarioStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.allSuscripcionUsuario"));
    updSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.updSuscripcion"));
    visitaSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.visitaSuscripcion"));
    updTipoSuscripcionStmt = connection.prepareStatement(Settings.get("sql.SuscripcionPersist.updTipoSuscripcion"));
  }

  private SuscripcionPersist() throws PersistException {

    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("SuscripcionPersist (SuscripcionPersist()): " + se.getMessage());
    }
  }

  private Suscripcion makeSuscripcion(ResultSet results)
    throws PersistException {
    try {
      Suscripcion suscrip = new Suscripcion();
      suscrip.setEnviar_mensajes(results.getString("ENVIAR_MENSAJES"));
      suscrip.setEstatus(results.getString("ESTATUS"));
      suscrip.setEmail_actualizaciones(results.getString("EMAIL_ACTUALIZACIONES"));
      suscrip.setEmail_nuevos(results.getString("EMAIL_NUEVOS"));
      suscrip.setEmail_anuncios(results.getString("EMAIL_ANUNCIOS"));
      suscrip.setTipo(results.getString("TIPO"));
      suscrip.setFecha_ultima_visita((java.util.Date) results.getTimestamp("FECHA_ULTIMA_VISITA"));
      suscrip.setId_foro(results.getInt("ID_FORO"));
      suscrip.setId_usuario(results.getInt("ID_USUARIO"));
      return suscrip;
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (makeSuscripcion()): " + e.getMessage());
    }
  }

  public String getRolSuscripcion(int id_foro, int id_usuario)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getTipoSuscripcionStmt) {
        getTipoSuscripcionStmt.clearParameters();
        getTipoSuscripcionStmt.setInt(1, id_foro);
        getTipoSuscripcionStmt.setInt(2, id_usuario);
        results = getTipoSuscripcionStmt.executeQuery();
      }
      if (!results.next()) {
        return Roles.NOT_MEMBER;
      } else {
        return Roles.getSuscriptionRole(results.getString("ESTATUS"),
          results.getString("TIPO"));
      }
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (getForumMemberRole()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Suscripcion get(int id_foro, int id_usuario)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getSuscripcionStmt) {
        getSuscripcionStmt.clearParameters();
        getSuscripcionStmt.setInt(1, id_foro);
        getSuscripcionStmt.setInt(2, id_usuario);
        results = getSuscripcionStmt.executeQuery();
      }
      if (results.next())
        return makeSuscripcion(results);
      else
        throw new UnknownPersistException("SuscripcionPersist (get()): "
          + "No se encontro el Registro Correspondiente al Usuario # "
          + id_usuario + " y el Foro # " + id_foro);
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (get()): " + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  private Suscripcion[] getSuscripcionesUsuario(int id_usuario)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection suscrips = new ArrayList();
      synchronized(getAllSuscripcionesUsuarioStmt) {
        getAllSuscripcionesUsuarioStmt.clearParameters();
        getAllSuscripcionesUsuarioStmt.setInt(1, id_usuario);
        results = getAllSuscripcionesUsuarioStmt.executeQuery();
      }
      while (results.next()) {
        suscrips.add(makeSuscripcion(results));
      }
      return (Suscripcion[])suscrips.toArray(new Suscripcion[0]);
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (getSuscripcionesUsuario()): " + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void removeSuscripcionesForo(int id_foro)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(remAllSuscripcionesForumStmt) {
        remAllSuscripcionesForumStmt.clearParameters();
        remAllSuscripcionesForumStmt.setInt(1, id_foro);
        int rowsChanged = remAllSuscripcionesForumStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist: (remove())" + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void remove(int id_foro, int id_usuario)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(remSuscripcionStmt) {
        remSuscripcionStmt.clearParameters();
        remSuscripcionStmt.setInt(1, id_foro);
        remSuscripcionStmt.setInt(2, id_usuario);
        int rowsChanged = remSuscripcionStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("SuscripcionPersist (remove()): "
            + "No se encontro el Registro Correspondiente al Usuario # "
            + id_usuario + " y el Foro # " + id_foro);
      }
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist: (remove())" + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void update(Suscripcion suscrip)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updSuscripcionStmt) {
        updSuscripcionStmt.clearParameters();
        updSuscripcionStmt.setString(1, suscrip.getEnviar_mensajes());
        updSuscripcionStmt.setString(2, suscrip.getEstatus());
        updSuscripcionStmt.setString(3, suscrip.getEmail_actualizaciones());
        updSuscripcionStmt.setString(4, suscrip.getEmail_nuevos());
        updSuscripcionStmt.setString(5, suscrip.getEmail_anuncios());
        updSuscripcionStmt.setString(6, suscrip.getTipo());
        updSuscripcionStmt.setTimestamp(7, new java.sql.Timestamp(suscrip.getFecha_ultima_visita().getTime()));
        updSuscripcionStmt.setInt(8, suscrip.getId_foro());
        updSuscripcionStmt.setInt(9, suscrip.getId_usuario());
        int rowsChanged = updSuscripcionStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("SuscripcionPersist (update()): "
            + "No se encontro el Registro Correspondiente al Usuario # "
            + suscrip.getId_usuario() + " y el Foro # " + suscrip.getId_foro());
      }
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (update()): " + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void put(Suscripcion suscrip) throws
    PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putSuscripcionStmt) {
        putSuscripcionStmt.clearParameters();
        putSuscripcionStmt.setString(1, suscrip.getEnviar_mensajes());
        putSuscripcionStmt.setString(2, suscrip.getEstatus());
        putSuscripcionStmt.setString(3, suscrip.getEmail_actualizaciones());
        putSuscripcionStmt.setString(4, suscrip.getEmail_nuevos());
        putSuscripcionStmt.setString(5, suscrip.getEmail_anuncios());
        putSuscripcionStmt.setString(6, suscrip.getTipo());
        putSuscripcionStmt.setTimestamp(7, new java.sql.Timestamp(suscrip.getFecha_ultima_visita().getTime()));
        putSuscripcionStmt.setInt(8, suscrip.getId_foro());
        putSuscripcionStmt.setInt(9, suscrip.getId_usuario());
        putSuscripcionStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
          throw new UnknownPersistException("SuscripcionPersist (put()): " + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void visitaSuscripcion(int id_foro, int id_usuario)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(visitaSuscripcionStmt) {
        visitaSuscripcionStmt.clearParameters();
        visitaSuscripcionStmt.setTimestamp(1, new java.sql.Timestamp(new java.util.Date().getTime()));
        visitaSuscripcionStmt.setInt(2, id_foro);
        visitaSuscripcionStmt.setInt(3, id_usuario);
        int rowsChanged = visitaSuscripcionStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("SuscripcionPersist (visitaSuscripcion()): " + "No se pudo registrar la visitas al Foro # "
            + id_foro + " correspondiente al Usuario # " + id_usuario);
      }
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (visitaForum()): " + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void updateTipoSuscripcion(String tipo, int id_foro, int id_usuario)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updTipoSuscripcionStmt) {
        updTipoSuscripcionStmt.clearParameters();
        updTipoSuscripcionStmt.setString(1, tipo);
        updTipoSuscripcionStmt.setInt(2, id_foro);
        updTipoSuscripcionStmt.setInt(3, id_usuario);
        int rowsChanged = updTipoSuscripcionStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("SuscripcionPersist (updateTipoSuscripcion()): "
            + "No se encontro el Registro Correspondiente al Usuario # "
            + id_usuario + " y el Foro # " + id_foro);
      }
    }
    catch (SQLException e) {
      throw new PersistException("SuscripcionPersist (updateTipoSuscripcion()): " + e.getMessage());
    }
    finally {
      ConnectionPool.free(confrompool);
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      Settings.getInstance().load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      SuscripcionPersist sp = SuscripcionPersist.getInstance();
      Suscripcion s = sp.get(1,1);
      System.out.println(s.getFecha_ultima_visita());
    } catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }
}
