// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.Topico;
import com.netapplications.webforums_jsp.util.Settings;
import com.netapplications.webforums_jsp.util.FileUtils;

public class TopicoPersist extends Object {
  private static TopicoPersist instance;

  private Connection connection;

  private PreparedStatement getTopicoStmt;
  private PreparedStatement putTopicoStmt;
  private PreparedStatement remTopicoStmt;
  private PreparedStatement updTopicoStmt;

  private PreparedStatement addMensajeTopicoStmt;
  private PreparedStatement decMensajeTopicoStmt;
  private PreparedStatement remRespuestaTopicoStmt;
  private PreparedStatement addTopicoForumStmt;
  private PreparedStatement remTopicoForumStmt;
  private PreparedStatement getNextTopicoStmt;
  private PreparedStatement getPrevTopicoStmt;
  private PreparedStatement getAllIdTopicosForumStmt;
  private PreparedStatement getIdTopicosForoStmt;
  private PreparedStatement getCountMensajesStmt;
  private PreparedStatement updCountMensajesStmt;

  private TopicoIDGenerator topidgen;

  public static TopicoPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new TopicoPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getNextTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.nextTopico"));
    getPrevTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.prevTopico"));
    getTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.getTopico"));
    putTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.putTopico"));
    remTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.remTopico"));
    updTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.updTopico"));
    addMensajeTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.incMensTopico"));
    decMensajeTopicoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.decMensTopico"));
    getIdTopicosForoStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.getIdTopicosForo"));
    getCountMensajesStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.getCountMensajes"));
    updCountMensajesStmt = connection.prepareStatement(Settings.get("sql.TopicoPersist.updCountMensajes"));
  }

  private TopicoPersist() throws PersistException {
    topidgen = TopicoIDGenerator.getInstance();
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("TopicoPersist [TopicoPersist()]: " + se.getMessage());
    }
  }

  private Topico makeTopico(ResultSet results)
    throws PersistException {
    try {
      Topico topico = new Topico();
      topico.setId_topico(results.getInt("ID_TOPICO"));
      topico.setId_foro(results.getInt("ID_FORO"));
      topico.setNombre(results.getString("NOMBRE"));
      topico.setNum_respuestas(results.getInt("NUM_RESPUESTAS"));
      topico.setId_usuario(results.getInt("ID_USUARIO"));
      topico.setFecha_creacion((java.util.Date) results.getTimestamp("FECHA_CREACION"));
      topico.setFecha_ultimo_mensaje((java.util.Date) results.getTimestamp("FECHA_ULTIMO_MENSAJE"));
      return topico;
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist [makeTopico()]: " + e.getMessage());
    }
  }

  public void fixCounts() throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized(getCountMensajesStmt) {
        results = getCountMensajesStmt.executeQuery();
      }
      synchronized(updCountMensajesStmt) {
        while (results.next()) {
          updCountMensajesStmt.clearParameters();
          updCountMensajesStmt.setInt(1, results.getInt(2));
          updCountMensajesStmt.setInt(2, results.getInt(1));
          updCountMensajesStmt.executeUpdate();
        }
      }
    } catch (SQLException e) {
      throw new PersistException("TopicoPersist (fixCounts()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Topico get(int id_topico)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getTopicoStmt) {
        getTopicoStmt.clearParameters();
        getTopicoStmt.setInt(1, id_topico);
        results = getTopicoStmt.executeQuery();
      }
      if (results.next())
        return makeTopico(results);
      else
        throw new UnknownPersistException("TopicoPersist [get()]: " + "No se encontro el topico # " + id_topico);
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist [get()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void update(Topico topico)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updTopicoStmt) {
        updTopicoStmt.clearParameters();
        updTopicoStmt.setInt(1, topico.getId_foro());
        updTopicoStmt.setString(2, topico.getNombre());
        updTopicoStmt.setInt(3, topico.getNum_respuestas());
        updTopicoStmt.setInt(4, topico.getId_usuario());
        if (topico.getFecha_creacion()!=null) {
          updTopicoStmt.setTimestamp(5, new java.sql.Timestamp(topico.getFecha_creacion().getTime()));
        } else {
          updTopicoStmt.setNull(5, java.sql.Types.DATE);
        }
        if (topico.getFecha_ultimo_mensaje()!=null) {
          updTopicoStmt.setTimestamp(6, new java.sql.Timestamp(topico.getFecha_ultimo_mensaje().getTime()));
        } else {
          updTopicoStmt.setNull(6, java.sql.Types.DATE);
        }
        updTopicoStmt.setInt(7, topico.getId_topico());
        int rowsChanged = updTopicoStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("TopicoPersist [update()]: " + "No se encontro el topico # " +
              topico.getId_topico());
      }
    }
    catch (SQLException e) {
      throw new PersistException("TopicoMensajeDB [update()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void put(Topico topico) throws PersistException {
    int new_id = topidgen.getNextID();
    boolean done = false;
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putTopicoStmt) {
        putTopicoStmt.clearParameters();
        topico.setId_topico(new_id);
        putTopicoStmt.setInt(1, topico.getId_topico());
        putTopicoStmt.setInt(2, topico.getId_foro());
        putTopicoStmt.setString(3, topico.getNombre());
        putTopicoStmt.setInt(4, topico.getNum_respuestas());
        putTopicoStmt.setInt(5, topico.getId_usuario());
        if (topico.getFecha_creacion()!=null) {
          putTopicoStmt.setTimestamp(6, new java.sql.Timestamp(topico.getFecha_creacion().getTime()));
        } else {
          putTopicoStmt.setNull(6, java.sql.Types.DATE);
        }
        if (topico.getFecha_ultimo_mensaje()!=null) {
          putTopicoStmt.setTimestamp(7, new java.sql.Timestamp(topico.getFecha_ultimo_mensaje().getTime()));
        } else {
          putTopicoStmt.setNull(7, java.sql.Types.DATE);
        }
        putTopicoStmt.executeUpdate();
      }
      done = true;
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist [put()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
    if (done) {
      ForoPersist.getInstance().incrementNumTopicosForo(topico.getId_foro());
    }
  }

  public void incrementNumMensajesTopico(int id_topico) throws PersistException {
    incrementNumMensajesTopico(id_topico, 1);
  }

  public void incrementNumMensajesTopico(int id_topico, int increment)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(addMensajeTopicoStmt) {
        addMensajeTopicoStmt.clearParameters();
        addMensajeTopicoStmt.setInt(1, increment);
        addMensajeTopicoStmt.setTimestamp(2, new java.sql.Timestamp(new java.util.Date().getTime()));
        addMensajeTopicoStmt.setInt(3, id_topico);
        int rowsChanged = addMensajeTopicoStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("TopicoPersist (incrementNumMensajesTopico()): " + "No se pudo incrementar el contador de "
            + "respuestas del topico # " + id_topico);
      }
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist (incrementNumMensajesTopico()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void decrementNumMensajesTopico(int id_topico) throws PersistException {
    decrementNumMensajesTopico(id_topico, 1);
  }

  public void decrementNumMensajesTopico(int id_topico, int decrement)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(decMensajeTopicoStmt) {
        decMensajeTopicoStmt.clearParameters();
        decMensajeTopicoStmt.setInt(1, decrement);
        decMensajeTopicoStmt.setInt(2, id_topico);
        decMensajeTopicoStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist (decrementNumMensajesTopico()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  private int deleteTopico(int id_topico) throws SQLException {
    synchronized(remTopicoStmt) {
      remTopicoStmt.clearParameters();
      remTopicoStmt.setInt(1, id_topico);
      return remTopicoStmt.executeUpdate();
    }
  }

  private void removeTopico(int id_topico) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      if (deleteTopico(id_topico) < 1) {
          throw new UnknownPersistException("TopicoPersist [removeTopico()]: " + "No se pudo borrar el topico # "
            + id_topico);
      }
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist [removeTopico()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  private Integer[] getIntegerArray(ResultSet results, int position) throws SQLException {
    Collection ints = new ArrayList();
    while (results.next()) {
      ints.add(new Integer(results.getInt(position)));
    }
    return (Integer[])ints.toArray(new Integer[0]);
  }

   public void removeTopicoCascade(int id_topico)
    throws PersistException {
    int id_foro = get(id_topico).getId_foro();
    MensajePersist.getInstance().removeAllMensajesTopico(id_topico);
    removeTopico(id_topico);
    FileUtils.delete(Settings.get("servlet.attachmentRealPath")+id_foro+"/"+id_topico);
    ForoPersist.getInstance().decrementNumTopicosForo(id_foro);
  }

  public void removeAllTopicosForo(int id_foro) throws PersistException {
    ResultSet results;
    Connection confrompool = null;
    Integer[] arrIds = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(getIdTopicosForoStmt) {
        getIdTopicosForoStmt.clearParameters();
        getIdTopicosForoStmt.setInt(1, id_foro);
        arrIds = getIntegerArray(getIdTopicosForoStmt.executeQuery(), 1);
      }
      ConnectionPool.free(confrompool);
      confrompool = null;
      for (int i=0; i<arrIds.length; i++) {
        removeTopicoCascade(arrIds[i].intValue());
      }
    }
    catch (SQLException e) {
      throw new PersistException("TopicoPersist [removeAllTopicosForo()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public int getNextTopico(int id_foro, int id_topico)
    throws PersistException {
    ResultSet results;
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(getNextTopicoStmt) {
        getNextTopicoStmt.clearParameters();
        getNextTopicoStmt.setInt(1, id_foro);
        getNextTopicoStmt.setInt(2, id_topico);
        results = getNextTopicoStmt.executeQuery();
      }
      if (results.next()) {
        return results.getInt(1);
      } else {
        throw new UnknownPersistException("TopicoPersist [getNextTopico()]: No more topics found");
      }
    } catch (SQLException e) {
      throw new PersistException("TopicoPersist [getNextTopico()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public int getPrevTopico(int id_foro, int id_topico)
    throws PersistException {
    ResultSet results;
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(getPrevTopicoStmt) {
        getPrevTopicoStmt.clearParameters();
        getPrevTopicoStmt.setInt(1, id_foro);
        getPrevTopicoStmt.setInt(2, id_topico);
        results = getPrevTopicoStmt.executeQuery();
      }
      if (results.next()) {
        return results.getInt(1);
      } else {
        throw new UnknownPersistException("TopicoPersist [getPrevTopico()]: No more topics found");
      }
    } catch (SQLException e) {
      throw new PersistException("TopicoPersist [getPrevTopico()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      Settings.getInstance().load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      TopicoPersist tmdb = TopicoPersist.getInstance();
      Topico topic = new Topico();
      topic.setId_foro(38);
      topic.setId_usuario(1);
      topic.setNombre("hola");
      topic.setFecha_creacion(new java.util.Date());
      topic.setNum_respuestas(0);
      topic.setFecha_ultimo_mensaje(null);
      tmdb.put(topic);
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}

