// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import com.netapplications.webforums_jsp.util.Settings;

public class UsuarioIDGenerator {
  private int increment = 1;
  private int max = 0;
  private int currentID = 0;

  private Connection connection;
  private static UsuarioIDGenerator instance;
  private PreparedStatement updSurrogateStmt;
  private PreparedStatement getSurrogateStmt;

  public static UsuarioIDGenerator getInstance() throws PersistException {
    instance = null;
    if (instance == null) {
      instance = new UsuarioIDGenerator();
    }
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getSurrogateStmt = connection.prepareStatement(Settings.get("sql.UsuarioIDGenerator.get"));
    updSurrogateStmt = connection.prepareStatement(Settings.get("sql.UsuarioIDGenerator.upd"));
  }

  private UsuarioIDGenerator() throws PersistException {
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("UsuarioIDGenerator [UsuarioIDGenerator()]: " + se.getMessage());
    }
  }

  public synchronized int getNextID() throws PersistException {
  	if (++currentID > max) {
      Connection confrompool = null;
      try {
        confrompool = ConnectionPool.get();
        if (confrompool != connection) {
          synchronized (connection) {
            connection = confrompool;
          }
          parseStatements();
        }
        synchronized(updSurrogateStmt) {
          updSurrogateStmt.clearParameters();
          updSurrogateStmt.setInt(1, increment);
          int rowsChanged = updSurrogateStmt.executeUpdate();
          if (rowsChanged < 1) {
            throw new UnknownPersistException("UsuarioIDGenerator (getNextID()): " + " Failed to update Surrogates");
          }
        }
        synchronized(getSurrogateStmt) {
          ResultSet results = getSurrogateStmt.executeQuery();
          if (results.next()) {
            currentID = results.getInt("NEXTIDSET") - increment;
            max = currentID + increment - 1;
          } else {
            throw new UnknownPersistException("UsuarioIDGenerator [getNextID()]: " + " Failed to read Surrogates");
          }
        }
      } catch (SQLException e) {
        throw new PersistException("UsuarioIDGenerator [getNextID()]: " + e.getMessage());
      }
      finally {
        if (confrompool!=null) {
          ConnectionPool.free(confrompool);
        }
      }
    }
	  return currentID;
  }

  public static void main(String[] argc) {
    try {
      Settings.getInstance().load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      UsuarioIDGenerator uid = UsuarioIDGenerator.getInstance();
      System.out.println(uid.getNextID());
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
