// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.Usuario;
import com.netapplications.webforums_jsp.util.Settings;

public class UsuarioPersist {
  private static UsuarioPersist instance;

  private Connection connection;
  private PreparedStatement getUsuarioStmt;
  private PreparedStatement putUsuarioStmt;
  private PreparedStatement remUsuarioStmt;
  private PreparedStatement updUsuarioStmt;
  private PreparedStatement getUsuarioLoginStmt;
  private PreparedStatement getAllReceiptsStmt;
  private UsuarioIDGenerator idgen;

  public static UsuarioPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new UsuarioPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getUsuarioStmt = connection.prepareStatement(Settings.get("sql.UsuarioPersist.get"));
    getUsuarioLoginStmt = connection.prepareStatement(Settings.get("sql.UsuarioPersist.getLogin"));
    putUsuarioStmt = connection.prepareStatement(Settings.get("sql.UsuarioPersist.put"));
    remUsuarioStmt = connection.prepareStatement(Settings.get("sql.UsuarioPersist.rem"));
    updUsuarioStmt = connection.prepareStatement(Settings.get("sql.UsuarioPersist.upd"));
    getAllReceiptsStmt = connection.prepareStatement(Settings.get("sql.UsuarioPersist.getAllReceipts"));
  }

  private UsuarioPersist() throws PersistException {
    idgen = UsuarioIDGenerator.getInstance();
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("UsuarioPersist [UsuarioPersist()]: " + se.getMessage());
    }
  }

  private Usuario makeUsuario(ResultSet results)
    throws PersistException {
    try {
      Usuario Usuario = new Usuario();
      Usuario.setId_usuario(results.getInt("ID_USUARIO"));
      Usuario.setNombre_corto(results.getString("NOMBRE_CORTO"));
      Usuario.setNombre_completo(results.getString("NOMBRE_COMPLETO"));
      Usuario.setPassword(results.getString("PASSWORD"));
      Usuario.setEmail(results.getString("EMAIL"));
      Usuario.setEstatus(results.getString("ESTATUS"));
      Usuario.setFecha_registro((java.util.Date) results.getTimestamp("FECHA_REGISTRO"));
      Usuario.setFecha_ultimo_acceso((java.util.Date) results.getTimestamp("FECHA_ULTIMO_ACCESO"));
      Usuario.setAdministrador(results.getString("ADMINISTRADOR"));
      return Usuario;
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist [makeUsuario()]: " + e.getMessage());
    }
  }



  public synchronized Usuario[] getUsuariosToSendEmail(int id_mensaje)
    throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection Usuarios = new ArrayList();
      synchronized(getAllReceiptsStmt) {
        getAllReceiptsStmt.clearParameters();
        getAllReceiptsStmt.setInt(1, id_mensaje);
        results = getAllReceiptsStmt.executeQuery();
      }
      while (results.next()) {
        Usuarios.add(makeUsuario(results));
      }
      return (Usuario[])Usuarios.toArray(new Usuario[0]);
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist (getUsuariosToSendEmail()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Usuario getUsuario(int id_usuario)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getUsuarioStmt) {
        getUsuarioStmt.clearParameters();
        getUsuarioStmt.setInt(1, id_usuario);
        results = getUsuarioStmt.executeQuery();
      }
      if (results.next())
        return makeUsuario(results);
      else
        throw new UnknownPersistException("UsuarioPersist [getUsuario()]: " + "No se encontro el Usuario # " + id_usuario);
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist [getUsuario()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public Usuario getUsuarioLogin(String nom_usuario, String PASSWORD)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized (getUsuarioStmt) {
        getUsuarioLoginStmt.clearParameters();
        getUsuarioLoginStmt.setString(1, nom_usuario);
        getUsuarioLoginStmt.setString(2, PASSWORD);
        results = getUsuarioLoginStmt.executeQuery();
      }
      if (results.next())
        return makeUsuario(results);
      else
        throw new UnknownPersistException("UsuarioPersist [getUsuarioLogin()]: " + "No se encontro el Usuario : " + nom_usuario + "  la contrasea es invlida");
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist [getUsuarioLogin()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void update(Usuario Usuario)
    throws UnknownPersistException, PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(updUsuarioStmt) {
        updUsuarioStmt.clearParameters();
        updUsuarioStmt.setString(1, Usuario.getNombre_corto());
        updUsuarioStmt.setString(2, Usuario.getNombre_completo());
        updUsuarioStmt.setString(3, Usuario.getPassword());
        updUsuarioStmt.setString(4, Usuario.getEmail());
        updUsuarioStmt.setString(5, Usuario.getEstatus());
        updUsuarioStmt.setTimestamp(6, new java.sql.Timestamp(Usuario.getFecha_registro().getTime()));
        updUsuarioStmt.setTimestamp(7, new java.sql.Timestamp(Usuario.getFecha_ultimo_acceso().getTime()));
        updUsuarioStmt.setString(8, Usuario.getAdministrador());
        updUsuarioStmt.setInt(9, Usuario.getId_usuario());
        int rowsChanged = updUsuarioStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("UsuarioPersist [update()]: " + "No se encontro el Usuario # " +
              Usuario.getId_usuario());
      }
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist [update()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void put(Usuario Usuario) throws PersistException {
    int new_id = idgen.getNextID();
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(putUsuarioStmt) {
        putUsuarioStmt.clearParameters();
        Usuario.setId_usuario(new_id);
        putUsuarioStmt.setInt(1, Usuario.getId_usuario());
        putUsuarioStmt.setString(2, Usuario.getNombre_corto());
        putUsuarioStmt.setString(3, Usuario.getNombre_completo());
        putUsuarioStmt.setString(4, Usuario.getPassword());
        putUsuarioStmt.setString(5, Usuario.getEmail());
        putUsuarioStmt.setString(6, Usuario.getEstatus());
        putUsuarioStmt.setTimestamp(7, new java.sql.Timestamp(new java.util.Date().getTime()));
        putUsuarioStmt.setTimestamp(8, new java.sql.Timestamp(new java.util.Date().getTime()));
        putUsuarioStmt.setString(9, Usuario.getAdministrador());
        putUsuarioStmt.executeUpdate();
      }
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist [put()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void removeUsuario(int id_usuario) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      synchronized(remUsuarioStmt) {
        remUsuarioStmt.clearParameters();
        remUsuarioStmt.setInt(1, id_usuario);
        int rowsChanged = remUsuarioStmt.executeUpdate();
        if (rowsChanged < 1)
          throw new UnknownPersistException("UsuarioPersist: " + "No se pudo borrar el Usuario # "
            + id_usuario);
      }
    }
    catch (SQLException e) {
      throw new PersistException("UsuarioPersist [removeUsuario()]: " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      Settings.getInstance().load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      String error = Settings.get("db.webforums_jsp.driver");
      System.out.println("jeje "+error);
      UsuarioPersist udb = UsuarioPersist.getInstance();
      System.out.println(udb.getUsuario(1).getNombre_completo());
    }
    catch (Exception e) {System.out.println("chin " + e.getMessage());}
  }
}