// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.VistaBusquedaTopico;
import com.netapplications.webforums_jsp.util.Settings;

public class VistaBusquedaTopicoPersist extends Object {
  private static VistaBusquedaTopicoPersist instance;

  private Connection connection;

  private PreparedStatement getTopicosForumStmt;
  private PreparedStatement getMensajesTopicoStmt;
  private PreparedStatement getNextMensajesStmt;

  public static VistaBusquedaTopicoPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new VistaBusquedaTopicoPersist();
    return instance;
  }

  private VistaBusquedaTopicoPersist() {
  }

  private VistaBusquedaTopico makeVistaBusquedaTopico(ResultSet results)
    throws PersistException {
    try {
      VistaBusquedaTopico vbTopico = new VistaBusquedaTopico();
      vbTopico.setId_foro(results.getInt("ID_FORO"));
      vbTopico.setNombre_foro(results.getString("NOMBRE_FORO"));
      vbTopico.setId_usuario(results.getInt("ID_USUARIO"));
      vbTopico.setNombre_usuario(results.getString("NOMBRE_USUARIO"));
      vbTopico.setId_topico(results.getInt("ID_TOPICO"));
      vbTopico.setNombre_topico(results.getString("NOMBRE_TOPICO"));
      vbTopico.setNum_respuestas(results.getInt("NUM_RESPUESTAS"));
      vbTopico.setFecha_creacion((java.util.Date) results.getTimestamp("FECHA_CREACION"));
      vbTopico.setFecha_ultimo_mensaje((java.util.Date) results.getTimestamp("FECHA_ULTIMO_MENSAJE"));
      return vbTopico;
    }
    catch (SQLException e) {
      throw new PersistException("TopicoTopicoDB [makeVistaBusquedaTopico()]: " + e.getMessage());
    }
  }

  public VistaBusquedaTopico[] findTopicos(String autor, String nombre_topico,
        boolean all_nombre_topico, String op_num_resp, String num_resp,
        java.util.Date fecha_creacion1, java.util.Date fecha_creacion2,
        java.util.Date fecha_ultmens1, java.util.Date fecha_ultmens2)
      throws PersistException {
    StringBuffer query = new StringBuffer(400);
    query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part1"));
    if (autor!=null && autor.length()>0) {
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part2"));
      query.append(autor);
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part3"));
      query.append(' ');
    }
    if (nombre_topico!=null && nombre_topico.length()>0) {
      StringTokenizer nomtopicotok = new StringTokenizer(nombre_topico,", ");
      if (!all_nombre_topico) {
        query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part4"));
      }
      boolean putAndOr = false;
      while (nomtopicotok.hasMoreTokens()) {
        if (all_nombre_topico) {
          query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part5"));
          query.append(' ');
        } else {
          if (putAndOr) {
            query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part6"));
            query.append(' ');
          } else {
            putAndOr = true;
          }
        }
        query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part7"));
        query.append(nomtopicotok.nextToken().toUpperCase());
        query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part8"));
        query.append(' ');
      }
      if (!all_nombre_topico) {
        query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part9"));
        query.append(' ');
      }
    }
    try {
      int dummy = Integer.parseInt(num_resp);
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part10"));
      query.append(op_num_resp);
      query.append(num_resp);
      query.append(' ');
    } catch (NumberFormatException e) {}
    if (fecha_creacion1!=null) {
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part11"));
      query.append(' ');
    }
    if (fecha_creacion2!=null) {
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part12"));
      query.append(' ');
    }
    if (fecha_ultmens1!=null) {
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part13"));
      query.append(' ');
    }
    if (fecha_ultmens2!=null) {
      query.append(Settings.get("sql.VistaBusquedaTopicoPersist.part14"));
    }
    try {
      connection = ConnectionPool.get();
      PreparedStatement findTopicos = connection.prepareStatement(query.toString());
      int aux=0;
      if (fecha_creacion1!=null) {
        findTopicos.setTimestamp(++aux, new java.sql.Timestamp(fecha_creacion1.getTime()));
      }
      if (fecha_creacion2!=null) {
        findTopicos.setTimestamp(++aux, new java.sql.Timestamp(fecha_creacion2.getTime()));
      }
      if (fecha_ultmens1!=null) {
        findTopicos.setTimestamp(++aux, new java.sql.Timestamp(fecha_ultmens1.getTime()));
      }
      if (fecha_ultmens2!=null) {
        findTopicos.setTimestamp(++aux, new java.sql.Timestamp(fecha_ultmens2.getTime()));
      }
      ResultSet results = findTopicos.executeQuery();
      Collection vbTopicos = new ArrayList();
      while (results.next()) {
        vbTopicos.add(makeVistaBusquedaTopico(results));
      }
      return (VistaBusquedaTopico[])vbTopicos.toArray(new VistaBusquedaTopico[0]);
    }
    catch (SQLException e) {
      throw new UnknownPersistException("VistaBusquedaTopicoPersist [findTopicos()]: " + e.getMessage()
        + " \r\n{" + query.toString() + "}");
    }
    catch (Exception e) {
      throw new PersistException("VistaBusquedaTopicoPersist [findTopicos()]: " + e.getMessage()
        + " \r\n{" + query.toString() + "}");
    } finally {
      if (connection != null) {
        ConnectionPool.free(connection);
      }
    }
  }

  public static void main(String[] argc) {
    try {
      VistaBusquedaTopicoPersist tmdb = VistaBusquedaTopicoPersist.getInstance();
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}

