// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.VistaMensaje;
import com.netapplications.webforums_jsp.domain.MensajeTree;
import com.netapplications.webforums_jsp.util.Settings;

public class VistaMensajePersist extends Object {
  private static VistaMensajePersist instance;

  private Connection connection;

  private PreparedStatement getStmt;
  private PreparedStatement getMensajesTopicoStmt;
  private PreparedStatement getNextMensajesStmt;

  public static VistaMensajePersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new VistaMensajePersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getStmt = connection.prepareStatement(Settings.get("sql.VistaMensajePersist.get"));
    getMensajesTopicoStmt = connection.prepareStatement(Settings.get("sql.VistaMensajePersist.getMensajesTopico"));
    getNextMensajesStmt = connection.prepareStatement(Settings.get("sql.VistaMensajePersist.getNextMensajes"));
  }

  private VistaMensajePersist() throws PersistException {
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("VistaMensajePersist [VistaMensajePersist()]: " + se.getMessage());
    }
  }

  private VistaMensaje makeVistaMensaje(ResultSet results)
    throws PersistException {
    try {
      VistaMensaje vMensaje = new VistaMensaje();
      vMensaje.setId_mensaje(results.getInt("ID_MENSAJE"));
      vMensaje.setId_mensaje_original(results.getInt("ID_MENSAJE_ORIGINAL"));
      vMensaje.setAsunto(results.getString("ASUNTO"));
      vMensaje.setContenido(results.getString("CONTENIDO"));
      vMensaje.setPrioridad(results.getInt("PRIORIDAD"));
      vMensaje.setFecha_envio((java.util.Date) results.getTimestamp("FECHA_ENVIO"));
      vMensaje.setEnvio_anonimo(results.getString("ENVIO_ANONIMO"));
      vMensaje.setEstatus_mensaje(results.getString("ESTATUS_MENSAJE"));
      vMensaje.setIncluir_original(results.getString("INCLUIR_ORIGINAL"));
      vMensaje.setId_usuario(results.getInt("ID_USUARIO"));
      vMensaje.setNombre_usuario(results.getString("NOMBRE_USUARIO"));
      vMensaje.setEmail_usuario(results.getString("EMAIL_USUARIO"));
      vMensaje.setId_emoticon(results.getInt("ID_EMOTICON"));
      vMensaje.setNombre_autor(results.getString("NOMBRE_AUTOR"));
      vMensaje.setUri_attachment(results.getString("URI_ATTACHMENT"));
      vMensaje.setNombre_topico(results.getString("NOMBRE_TOPICO"));
      return vMensaje;
    }
    catch (SQLException e) {
      throw new PersistException("VistaMensajePersist [makeVistaMensaje()]: " + e.getMessage());
    }
  }

  public VistaMensaje get(int id_mensaje) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection vMensajes = new ArrayList();
      synchronized(getStmt) {
        getStmt.clearParameters();
        getStmt.setInt(1, id_mensaje);
        results = getStmt.executeQuery();
      }
      if (results.next()) {
        return makeVistaMensaje(results);
      } else {
        throw new UnknownPersistException("VistaMensajePersist (get()): No se encontro el mens # " + id_mensaje);
      }
    }
    catch (SQLException e) {
      throw new PersistException("VistaMensajePersist (get()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public VistaMensaje[] getNextMensajes(int id_mensaje) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection vMensajes = new ArrayList();
      synchronized(getNextMensajesStmt) {
        getNextMensajesStmt.clearParameters();
        getNextMensajesStmt.setInt(1, id_mensaje);
        results = getNextMensajesStmt.executeQuery();
      }
      while (results.next()) {
        vMensajes.add(makeVistaMensaje(results));
      }
      return (VistaMensaje[])vMensajes.toArray(new VistaMensaje[0]);
    }
    catch (SQLException e) {
      throw new PersistException("VistaMensajePersist (getNextMensajes()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public MensajeTree getMensajesTopicoTree(int id_topico) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      synchronized(getMensajesTopicoStmt) {
        getMensajesTopicoStmt.clearParameters();
        getMensajesTopicoStmt.setInt(1, id_topico);
        results = getMensajesTopicoStmt.executeQuery();
      }
      MensajeTree mtree = new MensajeTree();
      while (results.next()) {
        mtree.add(makeVistaMensaje(results));
      }
      return mtree;
    }
    catch (NoSuchElementException e) {
          throw new PersistException("VistaMensajePersist (getMensajesTopicoTree()): No se pudo generar el arbol: "
             + e.getMessage());
    }
    catch (SQLException e) {
      throw new PersistException("VistaMensajePersist (getMensajesTopicoTree()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      VistaMensajePersist tmdb = VistaMensajePersist.getInstance();
      MensajeTree mt = tmdb.getMensajesTopicoTree(1);
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
