// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.VistaSuscripcion;
import com.netapplications.webforums_jsp.types.*;
import com.netapplications.webforums_jsp.util.Settings;

public class VistaSuscripcionPersist extends Object {

  private static VistaSuscripcionPersist instance;

  private Connection connection;

  private PreparedStatement getAllSuscripForoStmt;
  private PreparedStatement getSuscripActivasForoStmt;
  private PreparedStatement getSuscripActivasNormalSuspendForoStmt;

  public static VistaSuscripcionPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new VistaSuscripcionPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getAllSuscripForoStmt = connection.prepareStatement(Settings.get("sql.VistaSuscripcionPersist.getAllSuscripForo"));
    getSuscripActivasForoStmt = connection.prepareStatement(Settings.get("sql.VistaSuscripcionPersist.getSuscripActivasForo"));
    getSuscripActivasNormalSuspendForoStmt = connection.prepareStatement(Settings.get("sql.VistaSuscripcionPersist.getSuscripActivasNormalSuspendForo"));
  }

  private VistaSuscripcionPersist() throws PersistException {
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("VistaSuscripcionPersist [VistaSuscripcionPersist()]: " + se.getMessage());
    }
  }

  private VistaSuscripcion makeVistaSuscripcion(ResultSet results)
    throws PersistException {
    try {
      VistaSuscripcion vsuscrip = new VistaSuscripcion();
      vsuscrip.setEnviar_mensajes(results.getString("ENVIAR_MENSAJES"));
      vsuscrip.setEstatus(results.getString("ESTATUS"));
      vsuscrip.setEmail_actualizaciones(results.getString("EMAIL_ACTUALIZACIONES"));
      vsuscrip.setEmail_nuevos(results.getString("EMAIL_NUEVOS"));
      vsuscrip.setEmail_anuncios(results.getString("EMAIL_ANUNCIOS"));
      vsuscrip.setTipo(results.getString("TIPO"));
      vsuscrip.setFecha_ultima_visita((java.util.Date) results.getTimestamp("FECHA_ULTIMA_VISITA"));
      vsuscrip.setNombre_corto_usuario(results.getString("NOMBRE_CORTO_USUARIO"));
      vsuscrip.setNombre_completo_usuario(results.getString("NOMBRE_COMPLETO_USUARIO"));
      vsuscrip.setEmail_usuario(results.getString("EMAIL_USUARIO"));
      vsuscrip.setId_foro(results.getInt("ID_FORO"));
      vsuscrip.setId_usuario(results.getInt("ID_USUARIO"));
      return vsuscrip;
    }
    catch (SQLException e) {
      throw new PersistException("VistaSuscripcionPersist (makeSuscripcion()): " + e.getMessage());
    }
  }

  public VistaSuscripcion[] getAllForo(int id_foro) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection vsuscrip = new ArrayList();
      synchronized(getAllSuscripForoStmt) {
        getAllSuscripForoStmt.clearParameters();
        getAllSuscripForoStmt.setInt(1, id_foro);
        results = getAllSuscripForoStmt.executeQuery();
      }
      while (results.next()) {
        vsuscrip.add(makeVistaSuscripcion(results));
      }
      return (VistaSuscripcion[])vsuscrip.toArray(new VistaSuscripcion[0]);
    } catch (SQLException e) {
      throw new PersistException("VistaSuscripcionPersist (getAllForo()): " + e.getMessage());
    } finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public VistaSuscripcion[] getActivasForo(int id_foro) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection vsuscrip = new ArrayList();
      synchronized(getSuscripActivasForoStmt) {
        getSuscripActivasForoStmt.clearParameters();
        getSuscripActivasForoStmt.setInt(1, id_foro);
        results = getSuscripActivasForoStmt.executeQuery();
      }
      while (results.next()) {
        vsuscrip.add(makeVistaSuscripcion(results));
      }
      return (VistaSuscripcion[])vsuscrip.toArray(new VistaSuscripcion[0]);
    } catch (SQLException e) {
      throw new PersistException("VistaSuscripcionPersist (getActivasForo()): " + e.getMessage());
    } finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public VistaSuscripcion[] getActivasNormSuspForo(int id_foro) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection vsuscrip = new ArrayList();
      synchronized(getSuscripActivasNormalSuspendForoStmt) {
        getSuscripActivasNormalSuspendForoStmt.clearParameters();
        getSuscripActivasNormalSuspendForoStmt.setInt(1, id_foro);
        results = getSuscripActivasNormalSuspendForoStmt.executeQuery();
      }
      while (results.next()) {
        vsuscrip.add(makeVistaSuscripcion(results));
      }
      return (VistaSuscripcion[])vsuscrip.toArray(new VistaSuscripcion[0]);
    } catch (SQLException e) {
      throw new PersistException("VistaSuscripcionPersist (getActivasNormSuspForo()): " + e.getMessage());
    } finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      VistaSuscripcionPersist tmdb = VistaSuscripcionPersist.getInstance();
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
