// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.persist;

import java.sql.*;
import java.util.*;
import com.netapplications.webforums_jsp.domain.VistaTopico;
import com.netapplications.webforums_jsp.util.Settings;

public class VistaTopicoPersist extends Object {

  private static VistaTopicoPersist instance;

  private Connection connection;

  private PreparedStatement getTopicosForumStmt;

  public static VistaTopicoPersist getInstance()
    throws PersistException {
    if (instance == null)
      instance = new VistaTopicoPersist();
    return instance;
  }

  private synchronized void parseStatements() throws SQLException {
    getTopicosForumStmt = connection.prepareStatement(Settings.get("sql.VistaTopicoPersist.getTopicosForum"));
  }

  private VistaTopicoPersist() throws PersistException {
    try {
      connection = ConnectionPool.get();
      parseStatements();
      ConnectionPool.free(connection);
    }
    catch (SQLException se) {
      throw new PersistException("VistaTopicoPersist [VistaTopicoPersist()]: " + se.getMessage());
    }
  }

  private VistaTopico makeVistaTopico(ResultSet results)
    throws PersistException {
    try {
      VistaTopico vTopico = new VistaTopico();
      vTopico.setId_topico(results.getInt("ID_TOPICO"));
      vTopico.setNombre_topico(results.getString("NOMBRE_TOPICO"));
      vTopico.setNum_respuestas(results.getInt("NUM_RESPUESTAS"));
      vTopico.setId_usuario(results.getInt("ID_USUARIO"));
      vTopico.setNombre_usuario(results.getString("NOMBRE_USUARIO"));
      vTopico.setEmail_usuario(results.getString("EMAIL_USUARIO"));
      vTopico.setFecha_creacion((java.util.Date) results.getTimestamp("FECHA_CREACION"));
      vTopico.setFecha_ultimo_mensaje((java.util.Date) results.getTimestamp("FECHA_ULTIMO_MENSAJE"));
      return vTopico;
    }
    catch (SQLException e) {
      throw new PersistException("VistaTopicoPersist [makeVistaTopico()]: " + e.getMessage());
    }
  }

  public VistaTopico[] getTopicosForo(int id_foro) throws PersistException {
    Connection confrompool = null;
    try {
      confrompool = ConnectionPool.get();
      if (confrompool != connection) {
        synchronized (connection) {
          connection = confrompool;
        }
        parseStatements();
      }
      ResultSet results;
      Collection vwTopicos = new ArrayList();
      synchronized(getTopicosForumStmt) {
        getTopicosForumStmt.clearParameters();
        getTopicosForumStmt.setInt(1, id_foro);
        results = getTopicosForumStmt.executeQuery();
      }
      while (results.next()) {
        vwTopicos.add(makeVistaTopico(results));
      }
      return (VistaTopico[])vwTopicos.toArray(new VistaTopico[0]);
    }
    catch (SQLException e) {
      throw new PersistException("VistaTopicoPersist (getTopicosForo()): " + e.getMessage());
    }
    finally {
      if (confrompool!=null) {
        ConnectionPool.free(confrompool);
      }
    }
  }

  public void destroy() {
  }

  public static void main(String[] argc) {
    try {
      VistaTopicoPersist tmdb = VistaTopicoPersist.getInstance();
    }
    catch (Exception e) {System.out.println(e.getMessage());}
  }
}
