// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.servlets;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import com.netapplications.webforums_jsp.util.Settings;
import com.netapplications.webforums_jsp.control.*;
import com.netapplications.webforums_jsp.persist.*;
import com.netapplications.webforums_jsp.domain.*;
import com.netapplications.webforums_jsp.view.*;

public class MainServlet extends HttpServlet {

  private CommandsHashMap commands;
  private NavItemsVector navitems;
  private String error;
  private String jspdir;
  private String firstcmdkey = "go-home";
  private long refresh_time;
  private long last_refresh = 0;

  public void init(ServletConfig config) throws ServletException {
    super.init(config);
    initSettings();
    initServletVariables();
    initPersistCounters();
    initGlobalObjects();
    initCommands();
    initNavItems();
  }

  public String getServletInfo() {
    return "com.netapplications.webforums_jsp.servlets.MainServlet Information";
  }

  private synchronized void initSettings() throws ServletException {
    String path = getServletContext().getInitParameter("webforumsConfigFile");
    try {
      Settings.getInstance().load(path);
    } catch (IOException e) {
      throw new ServletException("initSettings: " + e.getMessage() + " {" + path + "}");
    }
  }

  private synchronized void initServletVariables() throws ServletException {
    try {
      error = Settings.get("servlet.errorPage");
      jspdir = Settings.get("servlet.jspDir");
      refresh_time = Long.parseLong(Settings.get("servlet.refreshTime"));
    } catch (Exception e) {
      throw new ServletException("initServletVariables: " + e.getMessage());
    }
  }

  private synchronized void initPersistCounters() throws ServletException {
    try {
      ForoPersist fp = ForoPersist.getInstance();
      fp.fixCounts();
      TopicoPersist tp = TopicoPersist.getInstance();
      tp.fixCounts();
    }
    catch (PersistException e) {
      throw new ServletException("initPersistCounters: " + e.getMessage());
    }
  }

  private synchronized void initGlobalObjects() throws ServletException {
    try {
      EmoticonPersist ep = EmoticonPersist.getInstance();
      Emoticon[] emotList = ep.getAllEmoticons();
      getServletContext().setAttribute("emoticons", emotList);
      ForoPersist fp = ForoPersist.getInstance();
      Foro[] forosList = fp.getForosActivos();
      getServletContext().setAttribute("foros", forosList);
    }
    catch (PersistException e) {
      throw new ServletException("initGlobalObjects: " + e.getMessage());
    }
  }

  private synchronized void initCommands() {
    commands = new CommandsHashMap();
    commands.putCommand("go-home", new NullCommand("home.jsp"));
    commands.putCommand("go-login", new NullCommand("login.jsp"));
    commands.putCommand("do-login", new GetUserFromLoginCommand("home.jsp","login.jsp"));
    commands.putCommand("do-logout", new EndSessionCommand("home.jsp"));
    commands.putCommand("go-profile-within-forum", new GetUserProfileCommand("viewuser.jsp","login.jsp"));
    commands.putCommand("go-profile", new GetUserProfileCommand("profile.jsp","login.jsp"));
    commands.putCommand("do-upd-profile", new UpdUserProfileCommand("home.jsp"));
    commands.putCommand("go-add-user", new EndSessionCommand("register.jsp"));
    commands.putCommand("do-add-user", new AddUserCommand("home.jsp","register.jsp"));
    commands.putCommand("go-forum", new GetAllTopicsOfForumCommand("forumtop.jsp"));
    commands.putCommand("go-forum-combo", new GetAllTopicsOfForumCommand("forumtop.jsp"));
    commands.putCommand("go-subscribe", new GetForumSubscriptionCommand("subscription.jsp"));
    commands.putCommand("do-subscribe", new UpdForumSubscriptionCommand("forumtop.jsp"));
    commands.putCommand("go-view-subscripcions", new GetAllSuscriptionsForumCommand("members.jsp"));
    commands.putCommand("go-upd-moderators", new GetAllActiveSuscriptionsForumCommand("moderators.jsp"));
    commands.putCommand("do-upd-moderators", new UpdSusbscriptionsTypeCommand("forumtop.jsp"));
    commands.putCommand("go-upd-suspended", new GetAllActiveNormSuspSuscriptionsForumCommand("suspended.jsp"));
    commands.putCommand("do-upd-suspended", new UpdSusbscriptionsTypeCommand("forumtop.jsp"));
    commands.putCommand("go-topic", new GetAllMessagesOfTopicCommand("topicthread.jsp","topicflat.jsp",true));
    commands.putCommand("go-topic-switch-view", new GetAllMessagesOfTopicCommand("topicthread.jsp","topicflat.jsp",false));
    commands.putCommand("go-add-forum", new NullCommand("newforum.jsp"));
    commands.putCommand("do-add-forum", new AddForumCommand("home.jsp"));
    commands.putCommand("go-add-topic", new NullCommand("newtopic.jsp"));
    commands.putCommand("go-find-topic", new NullCommand("findtopics.jsp"));
    commands.putCommand("do-find-topic", new FindTopicsCommand("topicsfound.jsp"));
    commands.putCommand("go-find-message", new NullCommand("findmessages.jsp"));
    commands.putCommand("do-find-message", new FindMessagesCommand("messagesfound.jsp"));
    commands.putCommand("do-del-message", new DelMessageCascadeCommand("topicthread.jsp","topicflat.jsp"));
    commands.putCommand("do-del-topic", new DelTopicCascadeCommand("forumtop.jsp",true));
    commands.putCommand("do-del-topic-found", new DelTopicCascadeCommand("findtopics.jsp",false));
    commands.putCommand("do-del-forum", new DelForumCascadeCommand("home.jsp"));
    commands.putCommand("do-edit-message", new UpdMessageContentCommand("topicthread.jsp","topicflat.jsp"));
    commands.putCommand("go-upd-emoticons", new NullCommand("emoticons.jsp"));
    commands.putCommand("do-del-emoticons", new DelEmoticonCascadeCommand("home.jsp"));
    commands.putCommand("do-upd-emoticons", new UpdEmoticonCommand("home.jsp"));
    commands.putCommand("go-next-topic", new FetchAnotherTopicOfForum("topicthread.jsp","topicflat.jsp",false));
    commands.putCommand("go-prev-topic", new FetchAnotherTopicOfForum("topicthread.jsp","topicflat.jsp",true));
  }

  private synchronized void initNavItems() {
    navitems = new NavItemsVector();
    navitems.addNavItem(
      new NavButton("go-home","home.gif","Ir a la pgina inicial","Inicio"),
      Roles.getRolesListExcluding(null),
      commands.getKeysListExcluding("go-home,do-login,do-logout,do-upd-profile,do-add-user,do-add-forum,do-del-forum,do-upd-emoticons,do-del-emoticons"));
    navitems.addNavItem(
      new NavButton("go-login","register.gif","Ir a la pgina de registro","Registro"),
      Roles.NOT_LOGGED,
      "go-home,do-logout,do-del-forum");
    navitems.addNavItem(
      new NavButton("go-profile","profile.gif","Ir a la pgina de mantenimiento de sus datos personales","Perfil"),
      Roles.getRolesListExcluding(Roles.NOT_LOGGED),
      "go-home,do-login,do-upd-profile,do-add-user,do-add-forum,do-del-forum,do-upd-emoticons,do-del-emoticons");
    navitems.addNavItem(
      new NavButton("go-add-forum","newtopic.gif","Agregar un nuevo foro","Nuevo"),
      Roles.ADMINISTRATOR,
      "go-home,do-login,do-upd-profile,do-add-user,do-add-forum,do-del-forum,do-upd-emoticons,do-del-emoticons");
    navitems.addNavItem(
      new NavButton("go-upd-emoticons","newtopic.gif","Editar el catalogo de emoticons","Emoticons"),
      Roles.ADMINISTRATOR,
      "go-home,do-login,do-upd-profile,do-add-user,do-add-forum,do-del-forum,do-upd-emoticons,do-del-emoticons");
    navitems.addNavItem(
      new NavButton("go-forum","up.gif","Regresar a la pgina inicial del foro","Foro"),
      Roles.getRolesListExcluding(null),
      "go-topic,go-topic-switch-view,go-add-topic,do-del-message,do-edit-message,go-prev-topic,go-next-topic,go-subscribe,go-upd-moderators,go-upd-suspended,go-profile-within-forum,go-view-subscripcions");
    navitems.addNavItem(
      new NavButton("go-prev-topic","prev.gif","Tpico anterior","Atras"),
      Roles.getRolesListExcluding(null),
      "go-topic,go-topic-switch-view,go-add-topic,do-del-message,do-edit-message,go-prev-topic,go-next-topic");
    navitems.addNavItem(
      new NavButton("go-next-topic","next.gif","Tpico siguiente","Adelante"),
      Roles.getRolesListExcluding(null),
      "go-topic,go-topic-switch-view,go-add-topic,do-del-message,do-edit-message,go-prev-topic,go-next-topic");
    navitems.addNavItem(
      new NavButton("go-subscribe","check.gif","Parmetros de Suscripcin","Suscripcin"),
      Roles.getRolesListExcluding(Roles.NOT_LOGGED),
      "go-forum,do-subscribe,do-upd-moderators,do-upd-suspended,do-del-topic");
    navitems.addNavItem(
      new NavButton("go-view-subscripcions","users.gif","Ver a todos los miembros del foro","Miembros"),
      Roles.getRolesListExcluding(null),
      "go-forum,do-subscribe,do-upd-moderators,do-upd-suspended,do-del-topic");
    navitems.addNavItem(
      new NavButton("go-upd-moderators","checkuser.gif","Definir a los moderadores del foro","Moderadores"),
      Roles.ADMINISTRATOR+","+Roles.ADMIN_MODERATOR,
      "go-forum,do-subscribe,do-upd-moderators,do-upd-suspended,do-del-topic");
    navitems.addNavItem(
      new NavButton("go-upd-suspended","checkuser.gif","Bloquear(Censurar)/Desbloquear usuarios","Bloqueo"),
      Roles.ADMINISTRATOR+","+Roles.ADMIN_MODERATOR+","+Roles.MODERATOR,
      "go-forum,do-subscribe,do-upd-moderators,do-upd-suspended,do-del-topic");
    navitems.addNavItem(
      new NavButton("go-add-topic","newtopic.gif","Agregar un nuevo tpico","Nuevo"),
      Roles.ADMINISTRATOR+","+Roles.ADMIN_MODERATOR+","+Roles.MODERATOR+","+Roles.MEMBER,
      "go-forum,do-subscribe,do-upd-moderators,do-upd-suspended,do-del-topic");
    navitems.addNavItem(
      new NavButton("go-topic-switch-view","viewmode.gif","Cambiar modo visualizacin lista/arbol","Modo"),
      Roles.getRolesListExcluding(null),
      "go-topic,go-topic-switch-view,do-del-message,do-edit-message,go-prev-topic,go-next-topic");
    navitems.addNavItem(
      new NavButton("go-find-topic","searchlist.gif","Bsqueda de topicos","Tpicos"),
      Roles.getRolesListExcluding(null),
      "go-home,do-login,do-upd-profile,do-add-user,do-add-forum,do-find-topic,do-del-forum,do-upd-emoticons,do-del-emoticons");
    navitems.addNavItem(
      new NavButton("go-find-message","searchlist.gif","Bsqueda de mensajes","Mensajes"),
      Roles.getRolesListExcluding(null),
      "go-home,do-login,do-upd-profile,do-add-user,do-add-forum,do-find-message,do-del-forum,do-upd-emoticons,do-del-emoticons");
    navitems.addNavItem(
      new NavComboBox("go-forum-combo", "id_foro", "Navegacin rpida entre foros", "Seleccione un Foro",
      new ForosICatalogProvider(getServletContext())),
      Roles.getRolesListExcluding(null),
      commands.getKeysListExcluding(null));
    navitems.addNavItem(
      new NavButton("do-logout","exit.gif","Finalizar su sesin en WebForums","Terminar"),
      Roles.getRolesListExcluding(Roles.NOT_LOGGED),
      "go-home,do-login,do-upd-profile,do-add-user,do-add-forum,do-del-forum,do-upd-emoticons,do-del-emoticons");
  }

  public void doGet(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
    Command cmd;
    String next;
    HttpSession session = req.getSession(true);
    if (session.getAttribute("usuario")==null ){
       session.setAttribute("role", Roles.NOT_LOGGED);
    }
    try {
      String cmdKey;
      if (req.getParameter("cmd") != null) {
        cmdKey = req.getParameter("cmd");
      } else {
        cmdKey = firstcmdkey;
      }
      cmd = commands.getCommand(cmdKey);
      next = cmd.execute(req, res);
      CommandToken.set(req);
      if (cmd.globalObjectUpdated() || last_refresh+refresh_time < (new Date()).getTime()) {
        initGlobalObjects();
        navitems.rebuildHTML();
        last_refresh = (new Date()).getTime();
      }
      req.setAttribute("navitems", navitems.getHTML((String) session.getAttribute("role"),cmdKey));
    }
    catch (CommandException e) {
      req.setAttribute("javax.servlet.jsp.jspException", e);
      next = error;
    }
    RequestDispatcher rd;
    rd = getServletContext().getRequestDispatcher(jspdir + next);
    rd.forward(req, res);
  }

  public void doPost(HttpServletRequest req, HttpServletResponse res) throws ServletException, IOException {
    if (req.getAttribute("flag_multipart")==null && req.getContentType()!=null && req.getContentType().toLowerCase().indexOf("multipart/form-data")>=0) {
      req.setAttribute("flag_multipart", "NOTNULL");
      RequestDispatcher rd;
      rd = getServletContext().getRequestDispatcher(Settings.get("servlet.addMessageServletAlias"));
      rd.forward(req, res);
    } else {
      doGet(req, res);
    }
  }
}
