// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.util;

import java.io.*;
import java.util.*;
import java.sql.*;
public class DBSettings implements SettingsSource {

  private Connection connection;
  private Properties prop = new Properties();
  private PreparedStatement updStmt;

  public DBSettings() {
    // default constructor
  }

  public synchronized void load (String param) throws IOException {
    String dbName = Settings.get(param + ".dbName");
    PreparedStatement getStmt;
    prop.clear();
    try {
      Class.forName(Settings.get("db." + dbName + ".driver"));
      connection = DriverManager.getConnection(Settings.get("db." + dbName + ".URL"),
                                               Settings.get("db." + dbName + ".user"),
                                               Settings.get("db." + dbName + ".password"));
      getStmt = connection.prepareStatement(Settings.get(param + ".loadSQL"));
      updStmt = connection.prepareStatement(Settings.get(param + ".saveSQL"));
      getStmt.clearParameters();
      ResultSet rs = getStmt.executeQuery();
      while (rs.next()) {
        prop.put(rs.getString(1),rs.getString(2));
      }
    } catch (ClassNotFoundException e) {
      throw new IOException("DBSettings [load()]: " + e.getMessage());
    } catch (SQLException e) {
      throw new IOException("DBSettings [load()]: " + e.getMessage());
    }
  }

  public synchronized void save () throws IOException {
    try {
      Enumeration enum = prop.propertyNames();
      while (enum.hasMoreElements()) {
        String name = (String) enum.nextElement();
        updStmt.clearParameters();
        updStmt.setString(1, prop.getProperty(name));
        updStmt.setString(2, name);
        updStmt.executeUpdate();
      }
    } catch (SQLException e) {
      throw new IOException("DBSettings [save()]: " + e.getMessage());
    }
  }

  public String getSetting(String key) {
    return prop.getProperty(key);
  }

  public void setSetting(String key,String value) {
    prop.setProperty(key,value);
  }

  public Enumeration getPropertyNames(){
    return prop.propertyNames();
  }

  public int size() {
    return prop.size();
  }

  public static void main(String[] argc) {
    try {
      Settings set = Settings.getInstance();
      set.load("C:\\JRun\\servers\\default\\default-app\\jforums\\settings\\init.conf");
//      set.setSetting("dbsettings.last_mailed_id_mensaje","45");
//      set.save();
      System.out.println(Settings.get("dbsettings.last_mailed_id_mensaje"));
    } catch (Exception e) {
      System.out.println("Ohhh nooo!" + e.getMessage());
    }
  }
}

