// Copyright (c) 2001 NetApplications
package com.netapplications.webforums_jsp.util;

import java.io.*;
import java.util.*;

public class Settings implements SettingsSource {
  private static Settings instance;
  private Properties base = new Properties();
  private Hashtable settings = new Hashtable();
  private String baseFile;
  
  private Settings() {
    // private constructor
  }
  
  public Settings(String baseFile) throws IOException {
    // needed???
    load(baseFile);
  }
  
  public static Settings getInstance(){
    if (instance == null){
      instance = new Settings();
    }
    return instance;
  }

  public synchronized void load (String file) throws IOException {


    // load as base properties
    // refresh 
    baseFile = file;
    settings = new Hashtable(); // reset
    File propFile=new File(file);
    base.load(new FileInputStream(propFile));
  }
  
  public synchronized void save (String key) throws IOException {
    SettingsSource temp = (SettingsSource)settings.get(key);
    if (temp!=null){
      temp.save();
    }
  }
  
  public synchronized void save () throws IOException {
    File propFile=new File(baseFile); 
    base.store(new FileOutputStream(propFile),"");
    // save DataSources
    Enumeration enum = settings.keys();
    while (enum.hasMoreElements()){
      String key = (String)enum.nextElement();
      save(key);
    }  
  }
  
  public String getSetting(String key) {
    String index= "";
    int pos;
    String value = null;
    
// System.out.println("Key: " + key);
    pos = key.indexOf(".");
    if (pos > -1){
      index = key.substring(0,pos);
    }
    if (index.equals("") ||
      index.equals("include") ||
      index.equals("directory")){
      //index = "base";
      value = base.getProperty(key);
 //                       System.out.println("No prelims, include or dir. Value of setting: " + value);
      return value;
    }else{
      key = key.substring(pos + 1);
    }
    
    SettingsSource temp = getSource(index);
    if (temp != null){
      value = temp.getSetting(key);
    }
 // System.out.println("Usual value: " + value);
    return value;
  }

  public static String get(String key){
    // convenience method as shortcut for:
    return getInstance().getSetting(key);
  }
  
  public void setSetting(String key, String value) {
    String index= "";
    int pos;
    pos = key.indexOf(".");
    if (pos > -1){
      index = key.substring(0,pos);
    }
    if (index.equals("") || 
      index.equals("include") ||
      index.equals("directory")){
      value = base.getProperty(key);
      return ;
    }else{
      key = key.substring(pos + 1);
    }
    
    SettingsSource temp = getSource(index);
    if (temp != null){
      temp.setSetting(key,value);
    }
  }

  public Enumeration getPropertyNames(){
    return base.propertyNames();
  }
  
  public Enumeration getPropertyNames(String sub){
    SettingsSource temp = getSource(sub);
    return temp.getPropertyNames();
  }
  
  public String translateDirectory(String path) {
    int pos, oldpos;
    StringBuffer out = new StringBuffer();
    oldpos = 0;
    
    if (path != null){
      pos = path.indexOf("%");
      while (pos > -1){
        out.append(path.substring(oldpos,pos));
        oldpos = pos;
        pos = path.indexOf("%",pos + 1);
        if (pos == -1){ pos = path.length();}
        String dir = path.substring(oldpos + 1, pos);
        out.append(getSetting("directory."+dir));
        oldpos = pos+1;
        pos = path.indexOf("%",pos + 1);
      }
      out.append(path.substring(oldpos,path.length()));
    }
    return out.toString();
  }   
  
  public int size() {
    if (settings != null)
      return settings.size();
    else return 0;
  }


  private SettingsSource getSource(String index){
    SettingsSource temp = (SettingsSource)settings.get(index);
    if (temp == null){
// System.out.println("getSource: Temp is null - need to load " + index);
      String param = base.getProperty("include."+index+".param");
// System.out.println("getSource: Temp is null - found: " + param);
      String sourceType = base.getProperty("include."+index+".sourceType");
      if (sourceType == null){
        sourceType = "com.netapplications.webforums_jsp.util.FileSettings";
      }
      try{
// System.out.println("getSource: About to try getting FileSettings class: " + sourceType);
        temp = (SettingsSource)Class.forName(sourceType).newInstance();
// System.out.println("getSource: New class created.");
        temp.load(param);
// System.out.println("getSource: temp loaded with param: " + temp.getSetting(param));
        settings.put(index,temp);
      }
      catch (IOException e){
        //nevermind, use base
      }
      catch (IllegalAccessException e){
      }
      catch (InstantiationException e){
      }
      catch (ClassNotFoundException e){
      }
    }
    return temp;
  }

  public static void main(String[] args){
    // testing main
    Settings s = Settings.getInstance();
    try{
      s.load("C:\\JRun\\servers\\default\\default-app\\webforums_jsp\\settings\\init.conf");
      System.out.println("dir:" + s.translateDirectory("%conf%\\db.conf"));
      System.out.println("db.webforums_jsp.driver:" + Settings.get("db.webforums_jsp.driver"));
    }catch (IOException e){
      System.out.println("damn! $##$");
    }
  }
}
